@@ -5,9 +5,8 @@
 // is a cheap way of eliminating redundant work, as many entities can be skipped
 // with a single check. 
 // 
-// This example shows how to use change tracking in combination with a few other
-// techniques, like using prefabs to store a single dirty state for multiple
-// entities and instanced queries.
+// This example shows how to use change tracking in combination with using 
+// prefabs to store a single dirty state for multiple entities.
 
 typedef struct {
     bool value;
@@ -49,8 +48,7 @@ int main(int argc, char *argv[]) {
                 .trav = EcsIsA
             },
             { .id = ecs_id(Position) }
-        },
-        .flags = EcsQueryIsInstanced // Instanced iteration is faster (see example)
+        }
     });
 
     // Create two prefabs with a Dirty component. We can use this to share a
