@@ -25,18 +25,18 @@ inline metrics::metrics(flecs::world& world) {
 }
 
 inline metric_builder::~metric_builder() {
-    if (!m_created) {
-        ecs_metric_init(m_world, &m_desc);
+    if (!created_) {
+        ecs_metric_init(world_, &desc_);
     }
 }
 
 inline metric_builder& metric_builder::member(const char *name) {
     flecs::entity m;
-    if (m_desc.id) {
-        flecs::entity_t type = ecs_get_typeid(m_world, m_desc.id);
-        m = flecs::entity(m_world, type).lookup(name);
+    if (desc_.id) {
+        flecs::entity_t type = ecs_get_typeid(world_, desc_.id);
+        m = flecs::entity(world_, type).lookup(name);
     } else {
-        m = flecs::world(m_world).lookup(name);
+        m = flecs::world(world_).lookup(name);
     }
     if (!m) {
         flecs::log::err("member '%s' not found", name);
@@ -46,7 +46,7 @@ inline metric_builder& metric_builder::member(const char *name) {
 
 template <typename T>
 inline metric_builder& metric_builder::member(const char *name) {
-    flecs::entity e (m_world, _::cpp_type<T>::id(m_world));
+    flecs::entity e (world_, _::type<T>::id(world_));
     flecs::entity_t m = e.lookup(name);
     if (!m) {
         flecs::log::err("member '%s' not found in type '%s'", 
@@ -57,32 +57,32 @@ inline metric_builder& metric_builder::member(const char *name) {
 }
 
 inline metric_builder& metric_builder::dotmember(const char *expr) {
-    m_desc.dotmember = expr;
+    desc_.dotmember = expr;
     return *this;
 }
 
 template <typename T>
 inline metric_builder& metric_builder::dotmember(const char *expr) {
-    m_desc.dotmember = expr;
-    m_desc.id = _::cpp_type<T>::id(m_world);
+    desc_.dotmember = expr;
+    desc_.id = _::type<T>::id(world_);
     return *this;
 }
 
 inline metric_builder::operator flecs::entity() {
-    if (!m_created) {
-        m_created = true;
-        flecs::entity result(m_world, ecs_metric_init(m_world, &m_desc));
-        m_desc.entity = result;
+    if (!created_) {
+        created_ = true;
+        flecs::entity result(world_, ecs_metric_init(world_, &desc_));
+        desc_.entity = result;
         return result;
     } else {
-        return flecs::entity(m_world, m_desc.entity);
+        return flecs::entity(world_, desc_.entity);
     }
 }
 
 template <typename... Args>
 inline flecs::metric_builder world::metric(Args &&... args) const {
-    flecs::entity result(m_world, FLECS_FWD(args)...);
-    return flecs::metric_builder(m_world, result);
+    flecs::entity result(world_, FLECS_FWD(args)...);
+    return flecs::metric_builder(world_, result);
 }
 
 template <typename Kind>
@@ -91,8 +91,8 @@ inline untyped_component& untyped_component::metric(
     const char *brief, 
     const char *metric_name) 
 {
-    flecs::world w(m_world);
-    flecs::entity e(m_world, m_id);
+    flecs::world w(world_);
+    flecs::entity e(world_, id_);
 
     const flecs::member_t *m = ecs_cpp_last_member(w, e);
     if (!m) {
