@@ -131,7 +131,7 @@ bool flecs_pipeline_check_term(
     (void)world;
 
     ecs_term_ref_t *src = &term->src;
-    if (term->inout == EcsInOutNone || term->inout == EcsInOutFilter) {
+    if (term->inout == EcsInOutFilter) {
         return false;
     }
 
@@ -684,9 +684,9 @@ static
 void flecs_run_startup_systems(
     ecs_world_t *world)
 {
-    ecs_id_record_t *idr = flecs_id_record_get(world, 
+    ecs_component_record_t *cdr = flecs_components_get(world, 
         ecs_dependson(EcsOnStart));
-    if (!idr || !flecs_table_cache_count(&idr->cache)) {
+    if (!cdr || !flecs_table_cache_count(&cdr->cache)) {
         /* Don't bother creating startup pipeline if no systems exist */
         return;
     }
@@ -842,7 +842,7 @@ ecs_entity_t ecs_pipeline_init(
     ecs_pipeline_state_t *pq = ecs_os_calloc_t(ecs_pipeline_state_t);
     pq->query = query;
     pq->match_count = -1;
-    pq->idr_inactive = flecs_id_record_ensure(world, EcsEmpty);
+    pq->idr_inactive = flecs_components_ensure(world, EcsEmpty);
     ecs_set(world, result, EcsPipeline, { pq });
 
     return result;
