@@ -169,71 +169,71 @@ bool flecs_scope_is_empty(
     ecs_script_scope_t *scope);
 
 ecs_script_scope_t* flecs_script_insert_scope(
-    ecs_script_parser_t *parser);
+    ecs_parser_t *parser);
 
 ecs_script_entity_t* flecs_script_insert_entity(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name,
     bool name_is_expr);
 
 ecs_script_pair_scope_t* flecs_script_insert_pair_scope(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *first,
     const char *second);
 
 ecs_script_with_t* flecs_script_insert_with(
-    ecs_script_parser_t *parser);
+    ecs_parser_t *parser);
 
 ecs_script_using_t* flecs_script_insert_using(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_module_t* flecs_script_insert_module(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_template_node_t* flecs_script_insert_template(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_annot_t* flecs_script_insert_annot(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name,
     const char *expr);
 
 ecs_script_var_node_t* flecs_script_insert_var(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_tag_t* flecs_script_insert_tag(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_tag_t* flecs_script_insert_pair_tag(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *first,
     const char *second);
 
 ecs_script_component_t* flecs_script_insert_component(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_component_t* flecs_script_insert_pair_component(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *first,
     const char *second);
 
 ecs_script_default_component_t* flecs_script_insert_default_component(
-    ecs_script_parser_t *parser);
+    ecs_parser_t *parser);
 
 ecs_script_var_component_t* flecs_script_insert_var_component(
-    ecs_script_parser_t *parser,
+    ecs_parser_t *parser,
     const char *name);
 
 ecs_script_if_t* flecs_script_insert_if(
-    ecs_script_parser_t *parser);
+    ecs_parser_t *parser);
 
 ecs_script_for_range_t* flecs_script_insert_for_range(
-    ecs_script_parser_t *parser);
+    ecs_parser_t *parser);
 
 #endif
