@@ -5,8 +5,8 @@
 
 #include "private_api.h"
 
-#ifdef FLECS_SCRIPT
-#include "addons/script/script.h"
+#ifdef FLECS_QUERY_DSL
+#include "addons/query_dsl/query_dsl.h"
 #endif
 
 bool ecs_id_match(
@@ -127,9 +127,9 @@ ecs_flags32_t ecs_id_get_flags(
     const ecs_world_t *world,
     ecs_id_t id)
 {
-    ecs_id_record_t *idr = flecs_id_record_get(world, id);
-    if (idr) {
-        return idr->flags;
+    ecs_component_record_t *cdr = flecs_components_get(world, id);
+    if (cdr) {
+        return cdr->flags;
     } else {
         return 0;
     }
@@ -139,7 +139,7 @@ ecs_id_t ecs_id_from_str(
     const ecs_world_t *world,
     const char *expr)
 {
-#ifdef FLECS_SCRIPT
+#ifdef FLECS_QUERY_DSL
     ecs_id_t result;
 
     /* Temporarily disable parser logging */
@@ -154,6 +154,7 @@ ecs_id_t ecs_id_from_str(
 #else
     (void)world;
     (void)expr;
-    ecs_abort(ECS_UNSUPPORTED, "ecs_id_from_str requires FLECS_SCRIPT addon");
+    ecs_abort(ECS_UNSUPPORTED, 
+        "ecs_id_from_str requires FLECS_QUERY_DSL addon");
 #endif
 }
