@@ -11,16 +11,6 @@ void flecs_iter_init(
     ecs_iter_t *it,
     ecs_flags8_t fields);
 
-void flecs_iter_validate(
-    ecs_iter_t *it);
-
-bool flecs_iter_next_row(
-    ecs_iter_t *it);
-
-bool flecs_iter_next_instanced(
-    ecs_iter_t *it,
-    bool result);
-
 void* flecs_iter_calloc(
     ecs_iter_t *it,
     ecs_size_t size,
