@@ -245,8 +245,8 @@ static
 void flecs_emit_propagate(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_id_record_t *idr,
-    ecs_id_record_t *tgt_idr,
+    ecs_component_record_t *cdr,
+    ecs_component_record_t *tgt_idr,
     ecs_entity_t trav,
     ecs_event_id_record_t **iders,
     int32_t ider_count);
@@ -255,8 +255,8 @@ static
 void flecs_emit_propagate_id(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_id_record_t *idr,
-    ecs_id_record_t *cur,
+    ecs_component_record_t *cdr,
+    ecs_component_record_t *cur,
     ecs_entity_t trav,
     ecs_event_id_record_t **iders,
     int32_t ider_count)
@@ -274,7 +274,7 @@ void flecs_emit_propagate_id(
             continue;
         }
 
-        bool owned = flecs_id_record_get_table(idr, table) != NULL;
+        bool owned = flecs_component_get_table(cdr, table) != NULL;
 
         int32_t e, entity_count = ecs_table_count(table);
         it->table = table;
@@ -305,11 +305,11 @@ void flecs_emit_propagate_id(
         for (e = 0; e < entity_count; e ++) {
             ecs_record_t *r = flecs_entities_get(world, entities[e]);
             ecs_assert(r != NULL, ECS_INTERNAL_ERROR, NULL);
-            ecs_id_record_t *idr_t = r->idr;
+            ecs_component_record_t *idr_t = r->cdr;
             if (idr_t) {
                 /* Only notify for entities that are used in pairs with
                  * traversable relationships */
-                flecs_emit_propagate(world, it, idr, idr_t, trav,
+                flecs_emit_propagate(world, it, cdr, idr_t, trav,
                     iders, ider_count);
             }
         }
@@ -323,8 +323,8 @@ static
 void flecs_emit_propagate(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_id_record_t *idr,
-    ecs_id_record_t *tgt_idr,
+    ecs_component_record_t *cdr,
+    ecs_component_record_t *tgt_idr,
     ecs_entity_t propagate_trav,
     ecs_event_id_record_t **iders,
     int32_t ider_count)
@@ -340,9 +340,9 @@ void flecs_emit_propagate(
     ecs_log_push_3();
 
     /* Propagate to records of traversable relationships */
-    ecs_id_record_t *cur = tgt_idr;
-    while ((cur = cur->trav.next)) {
-        cur->reachable.generation ++; /* Invalidate cache */
+    ecs_component_record_t *cur = tgt_idr;
+    while ((cur = flecs_component_trav_next(cur))) {
+        cur->pair->reachable.generation ++; /* Invalidate cache */
 
         /* Get traversed relationship */
         ecs_entity_t trav = ECS_PAIR_FIRST(cur->id);
@@ -353,7 +353,7 @@ void flecs_emit_propagate(
         }
 
         flecs_emit_propagate_id(
-            world, it, idr, cur, trav, iders, ider_count);
+            world, it, cdr, cur, trav, iders, ider_count);
     }
 
     ecs_log_pop_3();
@@ -362,7 +362,7 @@ void flecs_emit_propagate(
 static
 void flecs_emit_propagate_invalidate_tables(
     ecs_world_t *world,
-    ecs_id_record_t *tgt_idr)
+    ecs_component_record_t *tgt_idr)
 {
     ecs_assert(tgt_idr != NULL, ECS_INTERNAL_ERROR, NULL);
 
@@ -373,9 +373,9 @@ void flecs_emit_propagate_invalidate_tables(
     }
 
     /* Invalidate records of traversable relationships */
-    ecs_id_record_t *cur = tgt_idr;
-    while ((cur = cur->trav.next)) {
-        ecs_reachable_cache_t *rc = &cur->reachable;
+    ecs_component_record_t *cur = tgt_idr;
+    while ((cur = flecs_component_trav_next(cur))) {
+        ecs_reachable_cache_t *rc = &cur->pair->reachable;
         if (rc->current != rc->generation) {
             /* Subtree is already marked invalid */
             continue;
@@ -400,7 +400,7 @@ void flecs_emit_propagate_invalidate_tables(
 
             for (e = 0; e < entity_count; e ++) {
                 ecs_record_t *r = flecs_entities_get(world, entities[e]);
-                ecs_id_record_t *idr_t = r->idr;
+                ecs_component_record_t *idr_t = r->cdr;
                 if (idr_t) {
                     /* Only notify for entities that are used in pairs with
                      * traversable relationships */
@@ -427,7 +427,7 @@ void flecs_emit_propagate_invalidate(
             continue;
         }
 
-        ecs_id_record_t *idr_t = record->idr;
+        ecs_component_record_t *idr_t = record->cdr;
         if (idr_t) {
             /* Event is used as target in traversable relationship, propagate */
             flecs_emit_propagate_invalidate_tables(world, idr_t);
@@ -439,7 +439,7 @@ static
 void flecs_propagate_entities(
     ecs_world_t *world,
     ecs_iter_t *it,
-    ecs_id_record_t *idr,
+    ecs_component_record_t *cdr,
     const ecs_entity_t *entities,
     int32_t count,
     ecs_entity_t src,
@@ -466,13 +466,13 @@ void flecs_propagate_entities(
             continue;
         }
 
-        ecs_id_record_t *idr_t = record->idr;
+        ecs_component_record_t *idr_t = record->cdr;
         if (idr_t) {
             /* Entity is used as target in traversable pairs, propagate */
             ecs_entity_t e = src ? src : entities[i];
             it->sources[0] = e;
             flecs_emit_propagate(
-                world, it, idr, idr_t, 0, iders, ider_count);
+                world, it, cdr, idr_t, 0, iders, ider_count);
         }
     }
     
@@ -526,7 +526,7 @@ void* flecs_override(
     const ecs_type_t *emit_ids,
     ecs_id_t id,
     ecs_table_t *table,
-    ecs_id_record_t *idr)
+    ecs_component_record_t *cdr)
 {
     if (it->event != EcsOnAdd || (it->flags & EcsEventNoOnSet)) {
         return NULL;
@@ -545,7 +545,7 @@ void* flecs_override(
              * after it was inherited, as this does not change the actual value
              * of the component for the entity (it is copied from the existing
              * overridden component), and does not require an OnSet event. */
-            ecs_table_record_t *tr = flecs_id_record_get_table(idr, table);
+            const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
             if (!tr) {
                 continue;
             }
@@ -570,7 +570,7 @@ void flecs_emit_forward_up(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_id_record_t *idr,
+    ecs_component_record_t *cdr,
     ecs_vec_t *stack,
     ecs_vec_t *reachable_ids,
     int32_t depth);
@@ -583,13 +583,13 @@ void flecs_emit_forward_id(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_id_record_t *idr,
+    ecs_component_record_t *cdr,
     ecs_entity_t tgt,
     ecs_table_t *tgt_table,
     int32_t column,
     ecs_entity_t trav)
 {
-    ecs_id_t id = idr->id;
+    ecs_id_t id = cdr->id;
     ecs_entity_t event = er ? er->event : 0;
     bool inherit = trav == EcsIsA;
     bool may_override = inherit && (event == EcsOnAdd) && (emit_ids->count > 1);
@@ -608,6 +608,8 @@ void flecs_emit_forward_id(
         return;
     }
 
+    ecs_entity_t old_src = it->sources[0];
+
     it->ids[0] = id;
     it->sources[0] = tgt;
     it->event_id = id;
@@ -616,13 +618,13 @@ void flecs_emit_forward_id(
 
     int32_t storage_i = ecs_table_type_to_column_index(tgt_table, column);
     if (storage_i != -1) {
-        ecs_assert(idr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
+        ecs_assert(cdr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
         ecs_column_t *c = &tgt_table->data.columns[storage_i];
         it->trs[0] = &tgt_table->_->records[column];
         ECS_CONST_CAST(int32_t*, it->sizes)[0] = c->ti->size; /* safe, see above */
     }
 
-    ecs_table_record_t *tr = flecs_id_record_get_table(idr, table);
+    const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
     bool owned = tr != NULL;
 
     for (ider_i = 0; ider_i < ider_count; ider_i ++) {
@@ -641,9 +643,8 @@ void flecs_emit_forward_id(
 
         /* If component was added together with IsA relationship, still emit
          * OnSet event, as it's a new value for the entity. */
-        ecs_table_record_t *base_tr = ECS_CONST_CAST(
-                ecs_table_record_t*, it->trs[0]);
-        void *ptr = flecs_override(it, emit_ids, id, table, idr);
+        const ecs_table_record_t *base_tr = it->trs[0];
+        void *ptr = flecs_override(it, emit_ids, id, table, cdr);
         if (ptr) {
             override = true;
         }
@@ -674,6 +675,8 @@ void flecs_emit_forward_id(
         }
     }
 
+
+    it->sources[0] = old_src;
     it->up_fields = 0;
 }
 
@@ -685,7 +688,7 @@ void flecs_emit_forward_and_cache_id(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_id_record_t *idr,
+    ecs_component_record_t *cdr,
     ecs_entity_t tgt,
     ecs_record_t *tgt_record,
     ecs_table_t *tgt_table,
@@ -700,27 +703,27 @@ void flecs_emit_forward_and_cache_id(
     elem->tr = tgt_tr;
     elem->record = tgt_record;
     elem->src = tgt;
-    elem->id = idr->id;
+    elem->id = cdr->id;
 #ifndef NDEBUG
     elem->table = tgt_table;
 #endif
     ecs_assert(tgt_table == tgt_record->table, ECS_INTERNAL_ERROR, NULL);
 
-    flecs_emit_forward_id(world, er, er_onset, emit_ids, it, table, idr,
+    flecs_emit_forward_id(world, er, er_onset, emit_ids, it, table, cdr,
         tgt, tgt_table, column, trav);
 }
 
 static
 int32_t flecs_emit_stack_at(
     ecs_vec_t *stack,
-    ecs_id_record_t *idr)
+    ecs_component_record_t *cdr)
 {
     int32_t sp = 0, stack_count = ecs_vec_count(stack);
     ecs_table_t **stack_elems = ecs_vec_first(stack);
 
     for (sp = 0; sp < stack_count; sp ++) {
         ecs_table_t *elem = stack_elems[sp];
-        if (flecs_id_record_get_table(idr, elem)) {
+        if (flecs_component_get_table(cdr, elem)) {
             break;
         }
     }
@@ -731,9 +734,9 @@ int32_t flecs_emit_stack_at(
 static
 bool flecs_emit_stack_has(
     ecs_vec_t *stack,
-    ecs_id_record_t *idr)
+    ecs_component_record_t *cdr)
 {
-    return flecs_emit_stack_at(stack, idr) != ecs_vec_count(stack);
+    return flecs_emit_stack_at(stack, cdr) != ecs_vec_count(stack);
 }
 
 static
@@ -756,7 +759,7 @@ void flecs_emit_forward_cached_ids(
         ecs_reachable_elem_t *rc_elem = &elems[i];
         const ecs_table_record_t *rc_tr = rc_elem->tr;
         ecs_assert(rc_tr != NULL, ECS_INTERNAL_ERROR, NULL);
-        ecs_id_record_t *rc_idr = (ecs_id_record_t*)rc_tr->hdr.cache;
+        ecs_component_record_t *rc_idr = (ecs_component_record_t*)rc_tr->hdr.cache;
         ecs_record_t *rc_record = rc_elem->record;
 
         ecs_assert(rc_idr->id == rc_elem->id, ECS_INTERNAL_ERROR, NULL);
@@ -787,10 +790,16 @@ void flecs_emit_dump_cache(
         ecs_reachable_elem_t *elem = &elems[i];
         char *idstr = ecs_id_str(world, elem->id);
         char *estr = ecs_id_str(world, elem->src);
+        #ifndef NDEBUG
+        ecs_table_t *table = elem->table;
+        #else
+        ecs_table_t *table = NULL;
+        #endif
+        (void)table;
         ecs_dbg_3("- id: %s (%u), src: %s (%u), table: %p", 
             idstr, (uint32_t)elem->id,
             estr, (uint32_t)elem->src,
-            elem->table);
+            table);
         ecs_os_free(idstr);
         ecs_os_free(estr);
     }
@@ -810,7 +819,7 @@ void flecs_emit_forward_table_up(
     ecs_entity_t tgt,
     ecs_table_t *tgt_table,
     ecs_record_t *tgt_record,
-    ecs_id_record_t *tgt_idr,
+    ecs_component_record_t *tgt_idr,
     ecs_vec_t *stack,
     ecs_vec_t *reachable_ids,
     int32_t depth)
@@ -821,9 +830,10 @@ void flecs_emit_forward_table_up(
     int32_t rc_child_offset = ecs_vec_count(reachable_ids);
     int32_t stack_count = ecs_vec_count(stack);
 
-    /* If tgt_idr is out of sync but is not the current id record being updated,
+    /* If tgt_idr is out of sync but is not the current component record being updated,
      * keep track so that we can update two records for the cost of one. */
-    ecs_reachable_cache_t *rc = &tgt_idr->reachable;
+    ecs_assert(tgt_idr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_reachable_cache_t *rc = &tgt_idr->pair->reachable;
     bool parent_revalidate = (reachable_ids != &rc->ids) && 
         (rc->current != rc->generation);
     if (parent_revalidate) {
@@ -845,14 +855,14 @@ void flecs_emit_forward_table_up(
     for (i = 0; i < id_count; i ++) {
         ecs_id_t id = ids[i];
         ecs_table_record_t *tgt_tr = &tgt_table->_->records[i];
-        ecs_id_record_t *idr = (ecs_id_record_t*)tgt_tr->hdr.cache;
-        if (inherit && !(idr->flags & EcsIdOnInstantiateInherit)) {
+        ecs_component_record_t *cdr = (ecs_component_record_t*)tgt_tr->hdr.cache;
+        if (inherit && !(cdr->flags & EcsIdOnInstantiateInherit)) {
             continue;
         }
 
-        if (idr == tgt_idr) {
-            char *idstr = ecs_id_str(world, idr->id);
-            ecs_assert(idr != tgt_idr, ECS_CYCLE_DETECTED, idstr);
+        if (cdr == tgt_idr) {
+            char *idstr = ecs_id_str(world, cdr->id);
+            ecs_assert(cdr != tgt_idr, ECS_CYCLE_DETECTED, idstr);
             ecs_os_free(idstr);
             return;
         }
@@ -863,7 +873,8 @@ void flecs_emit_forward_table_up(
                 ecs_table_t*);
             t[0] = tgt_table;
 
-            ecs_reachable_cache_t *idr_rc = &idr->reachable;
+            ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+            ecs_reachable_cache_t *idr_rc = &cdr->pair->reachable;
             if (idr_rc->current == idr_rc->generation) {
                 /* Cache hit, use cached ids to prevent traversing the same
                  * hierarchy multiple times. This especially speeds up code 
@@ -881,7 +892,7 @@ void flecs_emit_forward_table_up(
                 /* Cache is dirty, traverse upwards */
                 do {
                     flecs_emit_forward_up(world, er, er_onset, emit_ids, it, 
-                        table, idr, stack, reachable_ids, depth);
+                        table, cdr, stack, reachable_ids, depth);
                     if (++i >= id_count) {
                         break;
                     }
@@ -897,15 +908,15 @@ void flecs_emit_forward_table_up(
             continue;
         }
 
-        int32_t stack_at = flecs_emit_stack_at(stack, idr);
+        int32_t stack_at = flecs_emit_stack_at(stack, cdr);
         if (parent_revalidate && (stack_at == (stack_count - 1))) {
-            /* If parent id record needs to be revalidated, add id */
+            /* If parent component record needs to be revalidated, add id */
             ecs_reachable_elem_t *elem = ecs_vec_append_t(a, &rc->ids, 
                 ecs_reachable_elem_t);
             elem->tr = tgt_tr;
             elem->record = tgt_record;
             elem->src = tgt;
-            elem->id = idr->id;
+            elem->id = cdr->id;
 #ifndef NDEBUG
             elem->table = tgt_table;
 #endif
@@ -917,7 +928,7 @@ void flecs_emit_forward_table_up(
         }
 
         flecs_emit_forward_and_cache_id(world, er, er_onset, emit_ids, it,
-            table, idr, tgt, tgt_record, tgt_table, tgt_tr, i, 
+            table, cdr, tgt, tgt_record, tgt_table, tgt_tr, i, 
                 reachable_ids, trav);
     }
 
@@ -961,19 +972,19 @@ void flecs_emit_forward_up(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_id_record_t *idr,
+    ecs_component_record_t *cdr,
     ecs_vec_t *stack,
     ecs_vec_t *reachable_ids,
     int32_t depth)
 {
     if (depth >= FLECS_DAG_DEPTH_MAX) {
-        char *idstr = ecs_id_str(world, idr->id);
+        char *idstr = ecs_id_str(world, cdr->id);
         ecs_assert(depth < FLECS_DAG_DEPTH_MAX, ECS_CYCLE_DETECTED, idstr);
         ecs_os_free(idstr);
         return;
     }
 
-    ecs_id_t id = idr->id;
+    ecs_id_t id = cdr->id;
     ecs_entity_t tgt = ECS_PAIR_SECOND(id);
     tgt = flecs_entities_get_alive(world, tgt);
     ecs_assert(tgt != 0, ECS_INTERNAL_ERROR, NULL);
@@ -984,7 +995,7 @@ void flecs_emit_forward_up(
     }
 
     flecs_emit_forward_table_up(world, er, er_onset, emit_ids, it, table, 
-        tgt, tgt_table, tgt_record, idr, stack, reachable_ids, depth + 1);
+        tgt, tgt_table, tgt_record, cdr, stack, reachable_ids, depth + 1);
 }
 
 static
@@ -995,14 +1006,15 @@ void flecs_emit_forward(
     const ecs_type_t *emit_ids,
     ecs_iter_t *it,
     ecs_table_t *table,
-    ecs_id_record_t *idr)
+    ecs_component_record_t *cdr)
 {
-    ecs_reachable_cache_t *rc = &idr->reachable;
+    ecs_assert(cdr->pair != NULL, ECS_INTERNAL_ERROR, NULL);
+    ecs_reachable_cache_t *rc = &cdr->pair->reachable;
 
     if (rc->current != rc->generation) {
         /* Cache miss, iterate the tree to find ids to forward */
         if (ecs_should_log_3()) {
-            char *idstr = ecs_id_str(world, idr->id);
+            char *idstr = ecs_id_str(world, cdr->id);
             ecs_dbg_3("reachable cache miss for %s", idstr);
             ecs_os_free(idstr);
         }
@@ -1012,7 +1024,7 @@ void flecs_emit_forward(
         ecs_vec_init_t(&world->allocator, &stack, ecs_table_t*, 0);
         ecs_vec_reset_t(&world->allocator, &rc->ids, ecs_reachable_elem_t);
         flecs_emit_forward_up(world, er, er_onset, emit_ids, it, table, 
-            idr, &stack, &rc->ids, 0);
+            cdr, &stack, &rc->ids, 0);
         it->sources[0] = 0;
         ecs_vec_fini_t(&world->allocator, &stack, ecs_table_t*);
 
@@ -1034,13 +1046,13 @@ void flecs_emit_forward(
     } else {
         /* Cache hit, use cached values instead of walking the tree */
         if (ecs_should_log_3()) {
-            char *idstr = ecs_id_str(world, idr->id);
+            char *idstr = ecs_id_str(world, cdr->id);
             ecs_dbg_3("reachable cache hit for %s", idstr);
             ecs_os_free(idstr);
             flecs_emit_dump_cache(world, &rc->ids);
         }
 
-        ecs_entity_t trav = ECS_PAIR_FIRST(idr->id);
+        ecs_entity_t trav = ECS_PAIR_FIRST(cdr->id);
         ecs_reachable_elem_t *elems = ecs_vec_first_t(&rc->ids, 
             ecs_reachable_elem_t);
         int32_t i, count = ecs_vec_count(&rc->ids);
@@ -1048,7 +1060,7 @@ void flecs_emit_forward(
             ecs_reachable_elem_t *elem = &elems[i];
             const ecs_table_record_t *tr = elem->tr;
             ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
-            ecs_id_record_t *rc_idr = (ecs_id_record_t*)tr->hdr.cache;
+            ecs_component_record_t *rc_idr = (ecs_component_record_t*)tr->hdr.cache;
             ecs_record_t *r = elem->record;
 
             ecs_assert(rc_idr->id == elem->id, ECS_INTERNAL_ERROR, NULL);
@@ -1069,7 +1081,7 @@ void flecs_emit_forward(
         entities = ECS_ELEM_T(entities, ecs_entity_t, it->offset);
         for (i = 0; i < it->count; i ++) {
             ecs_record_t *r = flecs_entities_get(world, entities[i]);
-            if (r->idr) {
+            if (r->cdr) {
                 break;
             }
         }
@@ -1082,7 +1094,7 @@ void flecs_emit_forward(
                 ecs_reachable_elem_t *elem = &elems[i];
                 const ecs_table_record_t *tr = elem->tr;
                 ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
-                ecs_id_record_t *rc_idr = (ecs_id_record_t*)tr->hdr.cache;
+                ecs_component_record_t *rc_idr = (ecs_component_record_t*)tr->hdr.cache;
                 ecs_record_t *r = elem->record;
 
                 ecs_assert(rc_idr->id == elem->id, ECS_INTERNAL_ERROR, NULL);
@@ -1093,7 +1105,7 @@ void flecs_emit_forward(
                 (void)r;
 
                 /* If entities already have the component, don't propagate */
-                if (flecs_id_record_get_table(rc_idr, it->table)) {
+                if (flecs_component_get_table(rc_idr, it->table)) {
                     continue;
                 }
 
@@ -1116,7 +1128,7 @@ void flecs_emit_forward(
 void flecs_emit(
     ecs_world_t *world,
     ecs_world_t *stage,
-    ecs_flags64_t set_mask,
+    ecs_flags64_t *set_mask,
     ecs_event_desc_t *desc)
 {
     flecs_poly_assert(world, ecs_world_t);
@@ -1246,7 +1258,7 @@ repeat_event:
          * One example is when an id was added with a "With" property, or 
          * inheriting from a prefab with overrides. In these cases an entity is 
          * moved directly to the archetype with the additional components. */
-        ecs_id_record_t *idr = NULL;
+        ecs_component_record_t *cdr = NULL;
         const ecs_type_info_t *ti = NULL;
         ecs_id_t id = id_array[i];
         ecs_assert(id == EcsAny || !ecs_id_is_wildcard(id), 
@@ -1258,22 +1270,27 @@ repeat_event:
         ecs_table_record_t *base_tr = NULL;
         ecs_entity_t base = 0;
         bool id_can_override = can_override;
-        ecs_flags64_t id_bit = 1llu << i;
-        if (id_bit & set_mask) {
-            /* Component is already set, so don't override with prefab value */
-            id_can_override = false;
+
+        /* Only added components can trigger overriding */
+        if (set_mask && event == EcsOnAdd) {
+            ecs_assert(i < 256, ECS_UNSUPPORTED, 
+                "cannot add more than 256 components in a single operation");
+            if (set_mask[i >> 6] & (1llu << (i & 63))) {
+                /* Component is already set, so don't override with prefab value */
+                id_can_override = false;
+            }
         }
 
         /* Check if this id is a pair of an traversable relationship. If so, we 
          * may have to forward ids from the pair's target. */
         if ((can_forward && is_pair) || id_can_override) {
-            idr = flecs_id_record_get(world, id);
-            if (!idr) {
+            cdr = flecs_components_get(world, id);
+            if (!cdr) {
                 /* Possible for union ids */
                 continue;
             }
 
-            ecs_flags32_t idr_flags = idr->flags;
+            ecs_flags32_t idr_flags = cdr->flags;
 
             if (is_pair && (idr_flags & EcsIdTraversable)) {
                 const ecs_event_record_t *er_fwd = NULL;
@@ -1301,22 +1318,22 @@ repeat_event:
                 }
 
                 /* Forward events for components from pair target */
-                flecs_emit_forward(world, er, er_fwd, ids, &it, table, idr);
+                flecs_emit_forward(world, er, er_fwd, ids, &it, table, cdr);
                 ecs_assert(it.event_cur == evtx, ECS_INTERNAL_ERROR, NULL);
             }
 
             if (id_can_override && !(idr_flags & EcsIdOnInstantiateDontInherit)) {
                 /* Initialize overridden components with value from base */
-                ti = idr->type_info;
+                ti = cdr->type_info;
                 if (ti) {
                     int32_t base_column = ecs_search_relation(world, table, 
                         0, id, EcsIsA, EcsUp, &base, NULL, &base_tr);
                     if (base_column != -1) {
                         /* Base found with component */
                         ecs_table_t *base_table = base_tr->hdr.table;
-                        if (idr->flags & EcsIdIsSparse) {
+                        if (cdr->flags & EcsIdIsSparse) {
                             override_ptr = flecs_sparse_get_any(
-                                idr->sparse, 0, base);
+                                cdr->sparse, 0, base);
                         } else {
                             base_column = base_tr->column;
                             ecs_assert(base_column != -1, ECS_INTERNAL_ERROR, NULL);
@@ -1330,7 +1347,7 @@ repeat_event:
                         /* For ids with override policy, check if base was added 
                          * in same operation. This will determine later on 
                          * whether we need to emit an OnSet event. */
-                        if (!(idr->flags & 
+                        if (!(cdr->flags & 
                             (EcsIdOnInstantiateInherit|EcsIdOnInstantiateDontInherit))) {
                             int32_t base_i;
                             for (base_i = 0; base_i < id_count; base_i ++) {
@@ -1357,8 +1374,8 @@ repeat_event:
              * example, both observers for (ChildOf, p) and (ChildOf, *) would
              * match an event for (ChildOf, p). */
             ider_count = flecs_event_observers_get(er, id, iders);
-            idr = idr ? idr : flecs_id_record_get(world, id);
-            ecs_assert(idr != NULL, ECS_INTERNAL_ERROR, NULL);
+            cdr = cdr ? cdr : flecs_components_get(world, id);
+            ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
         }
 
         if (!ider_count && !override_ptr) {
@@ -1366,13 +1383,26 @@ repeat_event:
             continue;
         }
 
-        ecs_assert(idr != NULL, ECS_INTERNAL_ERROR, NULL);
-        ecs_table_record_t *tr = flecs_id_record_get_table(idr, table);
-        if (tr == NULL) {
-            /* When a single batch contains multiple add's for an exclusive
-             * relationship, it's possible that an id was in the added list
-             * that is no longer available for the entity. */
-            continue;
+        ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
+        const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
+        ecs_table_record_t dummy_tr = {
+            .hdr.cache = (ecs_table_cache_t*)cdr,
+            .hdr.table = table,
+            .index = -1,
+            .column = -1,
+            .count = 0
+        };
+
+        if (id != EcsAny) {
+            if (tr == NULL) {
+                /* When a single batch contains multiple add's for an exclusive
+                * relationship, it's possible that an id was in the added list
+                * that is no longer available for the entity. */
+                continue;
+            }
+        } else {
+            /* When matching Any the table may not have a record for it */
+            tr = &dummy_tr;
         }
 
         int32_t storage_i;
@@ -1383,20 +1413,20 @@ repeat_event:
 
         if (count) {
             storage_i = tr->column;
-            bool is_sparse = idr->flags & EcsIdIsSparse;
+            bool is_sparse = cdr->flags & EcsIdIsSparse;
 
             if (!ecs_id_is_wildcard(id) && (storage_i != -1 || is_sparse)) {
                 void *ptr;
-                ecs_size_t size = idr->type_info->size;
+                ecs_size_t size = cdr->type_info->size;
 
                 if (is_sparse) {
                     ecs_assert(count == 1, ECS_UNSUPPORTED, 
                         "events for multiple entities are currently unsupported"
                         " for sparse components");
                     ecs_entity_t e = ecs_table_entities(table)[offset];
-                    ptr = flecs_sparse_get(idr->sparse, 0, e);
+                    ptr = flecs_sparse_get(cdr->sparse, 0, e);
                 } else{
-                    ecs_assert(idr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
+                    ecs_assert(cdr->type_info != NULL, ECS_INTERNAL_ERROR, NULL);
                     ecs_column_t *c = &columns[storage_i];
                     ptr = ECS_ELEM(c->data, size, offset);
                 }
@@ -1430,7 +1460,9 @@ repeat_event:
                                     ECS_INTERNAL_ERROR, NULL);
                             }
                         }
-                    } else if (er_onset && it.other_table) {
+                    } else if (er_onset && it.other_table && 
+                            it.other_table->type.count) 
+                    {
                         /* If an override was removed, this re-exposes the
                          * overridden component. Because this causes the actual
                          * (now inherited) value of the component to change, an
@@ -1484,7 +1516,7 @@ repeat_event:
          * entity/entities for which the event was generated is used as such a
          * target, events must be propagated downwards. */
         flecs_propagate_entities(
-            world, &it, idr, it.entities, count, 0, iders, ider_count);
+            world, &it, cdr, it.entities, count, 0, iders, ider_count);
     }
 
     can_override = false; /* Don't override twice */
@@ -1522,12 +1554,7 @@ void ecs_emit(
         ecs_assert(desc->offset == 0, ECS_INVALID_PARAMETER, NULL);
         ecs_assert(desc->count == 0, ECS_INVALID_PARAMETER, NULL);
         ecs_record_t *r = flecs_entities_get(world, desc->entity);
-        ecs_table_t *table;
-        if (!r || !(table = r->table)) {
-            /* Empty entities can't trigger observers */
-            return;
-        }
-        desc->table = table;
+        desc->table = r->table;
         desc->offset = ECS_RECORD_TO_ROW(r->row);
         desc->count = 1;
     }
