@@ -49,7 +49,6 @@ ecs_flags32_t flecs_id_flag_for_event(
     }
     if (e == EcsWildcard) {
         return EcsIdHasOnAdd|EcsIdHasOnRemove|EcsIdHasOnSet|
-            EcsIdHasOnTableFill|EcsIdHasOnTableEmpty|
             EcsIdHasOnTableCreate|EcsIdHasOnTableDelete;
     }
     return 0;
@@ -77,9 +76,9 @@ void flecs_inc_observer_count(
 
         ecs_flags32_t flags = flecs_id_flag_for_event(event);
         if (flags) {
-            ecs_id_record_t *idr = flecs_id_record_get(world, id);
-            if (idr) {
-                idr->flags |= flags;
+            ecs_component_record_t *cdr = flecs_components_get(world, id);
+            if (cdr) {
+                cdr->flags |= flags;
             }
         }
     } else if (result == 0) {
@@ -91,9 +90,9 @@ void flecs_inc_observer_count(
 
         ecs_flags32_t flags = flecs_id_flag_for_event(event);
         if (flags) {
-            ecs_id_record_t *idr = flecs_id_record_get(world, id);
-            if (idr) {
-                idr->flags &= ~flags;
+            ecs_component_record_t *cdr = flecs_components_get(world, id);
+            if (cdr) {
+                cdr->flags &= ~flags;
             }
         }
 
@@ -130,10 +129,19 @@ void flecs_register_observer_for_id(
     ecs_term_t *term = &o->query->terms[0];
     ecs_entity_t trav = term->trav;
 
-    int i;
+    int i, j;
     for (i = 0; i < o->event_count; i ++) {
         ecs_entity_t event = flecs_get_observer_event(
             term, o->events[i]);
+        for (j = 0; j < i; j ++) {
+            if (event == flecs_get_observer_event(term, o->events[j])) {
+                break;
+            }
+        }
+        if (i != j) {
+            /* Duplicate event, ignore */
+            continue;
+        }
 
         /* Get observers for event */
         ecs_event_record_t *er = flecs_event_record_ensure(observable, event);
@@ -190,10 +198,19 @@ void flecs_unregister_observer_for_id(
     ecs_term_t *term = &o->query->terms[0];
     ecs_entity_t trav = term->trav;
 
-    int i;
+    int i, j;
     for (i = 0; i < o->event_count; i ++) {
         ecs_entity_t event = flecs_get_observer_event(
             term, o->events[i]);
+        for (j = 0; j < i; j ++) {
+            if (event == flecs_get_observer_event(term, o->events[j])) {
+                break;
+            }
+        }
+        if (i != j) {
+            /* Duplicate event, ignore */
+            continue;
+        }
 
         /* Get observers for event */
         ecs_event_record_t *er = flecs_event_record_get(observable, event);
@@ -272,99 +289,34 @@ bool flecs_ignore_observer(
     return result;
 }
 
-static
-void flecs_observer_invoke(
-    ecs_world_t *world,
-    ecs_iter_t *it,
-    ecs_observer_t *o,
-    ecs_iter_action_t callback,
-    int32_t term_index) 
-{
-    ecs_assert(it->callback != NULL, ECS_INVALID_PARAMETER, NULL);
-
-    if (ecs_should_log_3()) {
-        char *path = ecs_get_path(world, it->system);
-        ecs_dbg_3("observer: invoke %s", path);
-        ecs_os_free(path);
-    }
-
-    ecs_log_push_3();
-
-    ecs_entity_t old_system = flecs_stage_set_system(
-        world->stages[0], o->entity);
-    world->info.observers_ran_frame ++;
-
-    ecs_query_t *query = o->query;
-    ecs_assert(term_index < query->term_count, ECS_INTERNAL_ERROR, NULL);
-    ecs_term_t *term = &query->terms[term_index];
-    if (it->table && (term->oper != EcsNot)) {
-        ecs_assert((it->offset + it->count) <= ecs_table_count(it->table), 
-            ECS_INTERNAL_ERROR, NULL);
-    }
-
-    ecs_termset_t row_fields = it->row_fields;
-    it->row_fields = query->row_fields;
-
-    bool match_this = query->flags & EcsQueryMatchThis;
-    if (match_this) {
-        callback(it);
-        ecs_os_inc(&query->eval_count);
-    } else {
-        ecs_entity_t observer_src = ECS_TERM_REF_ID(&term->src);
-        if (observer_src && !(term->src.id & EcsIsEntity)) {
-            observer_src = 0;
-        }
-
-        const ecs_entity_t *entities = it->entities;
-        int32_t i, count = it->count;
-        ecs_entity_t src = it->sources[0];
-        it->count = 1;
-        for (i = 0; i < count; i ++) {
-            ecs_entity_t e = entities[i];
-            it->entities = &e;
-            if (!observer_src) {
-                callback(it);
-                ecs_os_inc(&query->eval_count);
-            } else if (observer_src == e) {
-                ecs_entity_t dummy = 0;
-                it->entities = &dummy;
-                if (!src) {
-                    it->sources[0] = e;
-                }
-
-                callback(it);
-                ecs_os_inc(&query->eval_count);
-                it->sources[0] = src;
-                break;
-            }
-        }
-
-        it->entities = entities;
-        it->count = count;
-    }
-
-    it->row_fields = row_fields;
-
-    flecs_stage_set_system(world->stages[0], old_system);
-
-    ecs_log_pop_3();
-}
-
 static
 void flecs_default_uni_observer_run_callback(ecs_iter_t *it) {
     ecs_observer_t *o = it->ctx;
     it->ctx = o->ctx;
     it->callback = o->callback;
+    o->callback(it);
+}
 
-    if (ecs_should_log_3()) {
-        char *path = ecs_get_path(it->world, it->system);
-        ecs_dbg_3("observer %s", path);
-        ecs_os_free(path);
+static
+void flecs_observer_invoke(
+    ecs_observer_t *o,
+    ecs_iter_t *it)
+{
+    if (o->run) {
+        it->next = flecs_default_next_callback;
+        it->callback = o->callback;
+        it->interrupted_by = 0;
+        if (flecs_observer_impl(o)->flags & EcsObserverBypassQuery) {
+            it->ctx = o;
+        } else {
+            it->ctx = o->ctx;
+        }
+        o->run(it);
+    } else {
+        ecs_iter_action_t callback = o->callback;
+        it->callback = callback;
+        callback(it);
     }
-
-    ecs_log_push_3();
-    flecs_observer_invoke(it->real_world, it, o, o->callback, 0);
-    ecs_log_pop_3();
 }
 
 static
@@ -386,6 +338,14 @@ void flecs_uni_observer_invoke(
         return;
     }
 
+    if (ecs_should_log_3()) {
+        char *path = ecs_get_path(world, it->system);
+        ecs_dbg_3("observer: invoke %s", path);
+        ecs_os_free(path);
+    }
+
+    ecs_log_push_3();
+
     ecs_observer_impl_t *impl = flecs_observer_impl(o);
     bool is_filter = term->inout == EcsInOutNone;
     ECS_BIT_COND(it->flags, EcsIterNoData, is_filter);
@@ -396,25 +356,75 @@ void flecs_uni_observer_invoke(
     it->term_index = impl->term_index;
     it->query = query;
     it->ref_fields = query->fixed_fields | query->row_fields;
+    ecs_termset_t row_fields = it->row_fields;
     it->row_fields = query->row_fields;
-    
+
     ecs_entity_t event = it->event;
     int32_t event_cur = it->event_cur;
     it->event = flecs_get_observer_event(term, event);
 
-    if (o->run) {
-        it->next = flecs_default_next_callback;
-        it->callback = flecs_default_uni_observer_run_callback;
-        it->ctx = o;
-        o->run(it);
+    ecs_entity_t old_system = flecs_stage_set_system(
+        world->stages[0], o->entity);
+
+    bool match_this = query->flags & EcsQueryMatchThis;
+
+    if (match_this) {
+        /* Invoke observer for $this field */
+        flecs_observer_invoke(o, it);
+        ecs_os_inc(&query->eval_count);
     } else {
-        ecs_iter_action_t callback = o->callback;
-        it->callback = callback;
-        flecs_observer_invoke(world, it, o, callback, 0);
+        /* Not a $this field, translate the iterator data from a $this field to
+         * a field with it->sources set. */
+        ecs_entity_t observer_src = ECS_TERM_REF_ID(&term->src);
+        ecs_assert(observer_src != 0, ECS_INTERNAL_ERROR, NULL);
+        const ecs_entity_t *entities = it->entities;
+        int32_t i, count = it->count;
+        ecs_entity_t src = it->sources[0];
+        ecs_table_t *old_table = it->table;
+
+        it->entities = NULL;
+        it->count = 0;
+        it->table = NULL;
+
+        /* Loop all entities for which the event was emitted. Usually this is
+         * just one, but it is possible to emit events for a table range. */
+        for (i = 0; i < count; i ++) {
+            ecs_entity_t e = entities[i];
+
+            /* Filter on the source of the observer field */
+            if (observer_src == e) {
+                if (!src) {
+                    /* Only overwrite source if event wasn't forwarded or
+                     * propagated from another entity. */
+                    it->sources[0] = e;
+                }
+
+                flecs_observer_invoke(o, it);
+                ecs_os_inc(&query->eval_count);
+
+                /* Restore source */
+                it->sources[0] = src;
+
+                /* Observer can only match one source explicitly, so we don't
+                 * have to check any other entities. */
+                break;
+            }
+        }
+
+        it->entities = entities;
+        it->count = count;
+        it->table = old_table;
     }
 
+    flecs_stage_set_system(world->stages[0], old_system);
+
     it->event = event;
     it->event_cur = event_cur;
+    it->row_fields = row_fields;
+
+    ecs_log_pop_3();
+
+    world->info.observers_ran_frame ++;
 }
 
 void flecs_observers_invoke(
@@ -588,8 +598,8 @@ void flecs_multi_observer_invoke_no_query(
     flecs_stage_set_system(world->stages[0], old_system);
 }
 
-/* For convenience, so applications can (in theory) use a single run callback 
- * that uses ecs_iter_next to iterate results */
+/* For convenience, so applications can use a single run callback that uses 
+ * ecs_iter_next to iterate results for systems/queries and observers. */
 bool flecs_default_next_callback(ecs_iter_t *it) {
     if (it->interrupted_by) {
         return false;
@@ -824,6 +834,11 @@ int flecs_multi_observer_init(
         child_desc.term_index_ = query->terms[i].field_index;
         *term = query->terms[i];
 
+        /* Don't create observers for non-$this terms */
+        if (!ecs_term_match_this(term) && term->src.id & EcsIsVariable) {
+            continue;
+        }
+
         int16_t oper = term->oper;
         ecs_id_t id = term->id;
 
@@ -859,8 +874,8 @@ int flecs_multi_observer_init(
             term->oper = EcsAnd;
             for (ti = 0; ti < ti_count; ti ++) {
                 ecs_id_t ti_id = ti_ids[ti];
-                ecs_id_record_t *idr = flecs_id_record_get(world, ti_id);
-                if (idr->flags & EcsIdOnInstantiateDontInherit) {
+                ecs_component_record_t *cdr = flecs_components_get(world, ti_id);
+                if (cdr->flags & EcsIdOnInstantiateDontInherit) {
                     continue;
                 }
 
@@ -973,6 +988,21 @@ ecs_observer_t* flecs_observer_init(
     ecs_check(o->query->term_count > 0, ECS_INVALID_PARAMETER,
         "observer must have at least one term");
 
+    int i, var_count = 0;
+    for (i = 0; i < o->query->term_count; i ++) {
+        ecs_term_t *term = &o->query->terms[i];
+        if (!ecs_term_match_this(term)) {
+            if (term->src.id & EcsIsVariable) {
+                /* Term has a non-$this variable source */
+                var_count ++;
+            }
+        }
+    }
+
+    ecs_check(o->query->term_count > var_count, ECS_UNSUPPORTED,
+        "observers with only non-$this variable sources are not yet supported");
+    (void)var_count;
+
     ecs_observable_t *observable = desc->observable;
     if (!observable) {
         observable = ecs_get_observable(world);
@@ -999,7 +1029,6 @@ ecs_observer_t* flecs_observer_init(
     /* Check if observer is monitor. Monitors are created as multi observers
      * since they require pre/post checking of the filter to test if the
      * entity is entering/leaving the monitor. */
-    int i;
     for (i = 0; i < FLECS_EVENT_DESC_MAX; i ++) {
         ecs_entity_t event = desc->events[i];
         if (!event) {
