@@ -27,6 +27,12 @@ extern const ecs_entity_t EcsFlag;
 #define ECS_MAX_JOBS_PER_WORKER (16)
 #define ECS_MAX_DEFER_STACK (8)
 
+/* The bitmask used when determining the table version array index */
+#define ECS_TABLE_VERSION_ARRAY_BITMASK (0xff)
+
+/* The number of table versions to split tables across */
+#define ECS_TABLE_VERSION_ARRAY_SIZE (ECS_TABLE_VERSION_ARRAY_BITMASK + 1)
+
 /* Magic number for a flecs object */
 #define ECS_OBJECT_MAGIC (0x6563736f)
 
@@ -95,6 +101,7 @@ typedef struct ecs_world_allocators_t {
     ecs_block_allocator_t graph_edge_lo;
     ecs_block_allocator_t graph_edge;
     ecs_block_allocator_t id_record;
+    ecs_block_allocator_t pair_id_record;
     ecs_block_allocator_t id_record_chunk;
     ecs_block_allocator_t table_diff;
     ecs_block_allocator_t sparse_chunk;
@@ -115,6 +122,7 @@ typedef struct ecs_stage_allocators_t {
 
 /** Types for deferred operations */
 typedef enum ecs_cmd_kind_t {
+    EcsCmdNew,
     EcsCmdClone,
     EcsCmdBulkNew,
     EcsCmdAdd,
@@ -156,7 +164,7 @@ typedef struct ecs_cmd_t {
     ecs_cmd_kind_t kind;             /* Command kind */
     int32_t next_for_entity;         /* Next operation for entity */    
     ecs_id_t id;                     /* (Component) id */
-    ecs_id_record_t *idr;            /* Id record (only for set/mut/emplace) */
+    ecs_component_record_t *cdr;            /* component record (only for set/mut/emplace) */
     ecs_cmd_entry_t *entry;
     ecs_entity_t entity;             /* Entity id */
 
@@ -168,13 +176,6 @@ typedef struct ecs_cmd_t {
     ecs_entity_t system;             /* System that enqueued the command */
 } ecs_cmd_t;
 
-/* Data structures that store the command queue */
-typedef struct ecs_commands_t {
-    ecs_vec_t queue;
-    ecs_stack_t stack;          /* Temp memory used by deferred commands */
-    ecs_sparse_t entries;       /* <entity, op_entry_t> - command batching */
-} ecs_commands_t;
-
 /** Callback used to capture commands of a frame */
 typedef void (*ecs_on_commands_action_t)(
     const ecs_stage_t *stage,
@@ -250,7 +251,7 @@ typedef struct ecs_monitor_set_t {
 
 /* Data stored for id marked for deletion */
 typedef struct ecs_marked_id_t {
-    ecs_id_record_t *idr;
+    ecs_component_record_t *cdr;
     ecs_id_t id;
     ecs_entity_t action;             /* Set explicitly for delete_with, remove_all */
     bool delete_id;
@@ -295,18 +296,18 @@ struct ecs_world_t {
     ecs_header_t hdr;
 
     /* --  Type metadata -- */
-    ecs_id_record_t **id_index_lo;
-    ecs_map_t id_index_hi;           /* map<id, ecs_id_record_t*> */
+    ecs_component_record_t **id_index_lo;
+    ecs_map_t id_index_hi;           /* map<id, ecs_component_record_t*> */
     ecs_map_t type_info;             /* map<type_id, type_info_t> */
 
     /* -- Cached handle to id records -- */
-    ecs_id_record_t *idr_wildcard;
-    ecs_id_record_t *idr_wildcard_wildcard;
-    ecs_id_record_t *idr_any;
-    ecs_id_record_t *idr_isa_wildcard;
-    ecs_id_record_t *idr_childof_0;
-    ecs_id_record_t *idr_childof_wildcard;
-    ecs_id_record_t *idr_identifier_name;
+    ecs_component_record_t *idr_wildcard;
+    ecs_component_record_t *idr_wildcard_wildcard;
+    ecs_component_record_t *idr_any;
+    ecs_component_record_t *idr_isa_wildcard;
+    ecs_component_record_t *idr_childof_0;
+    ecs_component_record_t *idr_childof_wildcard;
+    ecs_component_record_t *idr_identifier_name;
 
     /* -- Mixins -- */
     ecs_world_t *self;
@@ -315,6 +316,10 @@ struct ecs_world_t {
     /* Unique id per generated event used to prevent duplicate notifications */
     int32_t event_id;
 
+    /* Array of table versions used with component refs to determine if the 
+     * cached pointer is still valid. */
+    uint32_t table_version[ECS_TABLE_VERSION_ARRAY_SIZE];
+
     /* Is entity range checking enabled? */
     bool range_check_enabled;
 
