@@ -25,24 +25,24 @@ bool flecs_query_select_w_id(
     ecs_flags32_t filter_mask)
 {
     ecs_query_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
-    ecs_id_record_t *idr = op_ctx->idr;
-    ecs_table_record_t *tr;
+    ecs_component_record_t *cdr = op_ctx->cdr;
+    const ecs_table_record_t *tr;
     ecs_table_t *table;
 
     if (!redo) {
-        if (!idr || idr->id != id) {
-            idr = op_ctx->idr = flecs_id_record_get(ctx->world, id);
-            if (!idr) {
+        if (!cdr || cdr->id != id) {
+            cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
+            if (!cdr) {
                 return false;
             }
         }
 
         if (ctx->query->pub.flags & EcsQueryMatchEmptyTables) {
-            if (!flecs_table_cache_all_iter(&idr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_all_iter(&cdr->cache, &op_ctx->it)) {
                 return false;
             }
         } else {
-            if (!flecs_table_cache_iter(&idr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_iter(&cdr->cache, &op_ctx->it)) {
                 return false;
             }
         }
@@ -60,10 +60,10 @@ repeat:
         table = tr->hdr.table;
         flecs_query_var_set_range(op, op->src.var, table, 0, 0, ctx);
     } else {
-        tr = (ecs_table_record_t*)op_ctx->it.cur;
+        tr = (const ecs_table_record_t*)op_ctx->it.cur;
         ecs_assert(tr != NULL, ECS_INTERNAL_ERROR, NULL);
         table = tr->hdr.table;
-        op_ctx->column = flecs_query_next_column(table, idr->id, op_ctx->column);
+        op_ctx->column = flecs_query_next_column(table, cdr->id, op_ctx->column);
         op_ctx->remaining --;
     }
 
@@ -94,8 +94,8 @@ bool flecs_query_with(
     const ecs_query_run_ctx_t *ctx)
 {
     ecs_query_and_ctx_t *op_ctx = flecs_op_ctx(ctx, and);
-    ecs_id_record_t *idr = op_ctx->idr;
-    ecs_table_record_t *tr;
+    ecs_component_record_t *cdr = op_ctx->cdr;
+    const ecs_table_record_t *tr;
 
     ecs_table_t *table = flecs_query_get_table(op, &op->src, EcsQuerySrc, ctx);
     if (!table) {
@@ -104,14 +104,14 @@ bool flecs_query_with(
 
     if (!redo) {
         ecs_id_t id = flecs_query_op_get_id(op, ctx);
-        if (!idr || idr->id != id) {
-            idr = op_ctx->idr = flecs_id_record_get(ctx->world, id);
-            if (!idr) {
+        if (!cdr || cdr->id != id) {
+            cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
+            if (!cdr) {
                 return false;
             }
         }
 
-        tr = flecs_id_record_get_table(idr, table);
+        tr = flecs_component_get_table(cdr, table);
         if (!tr) {
             return false;
         }
@@ -127,7 +127,7 @@ bool flecs_query_with(
             return false;
         }
 
-        op_ctx->column = flecs_query_next_column(table, idr->id, op_ctx->column);
+        op_ctx->column = flecs_query_next_column(table, cdr->id, op_ctx->column);
         ecs_assert(op_ctx->column != -1, ECS_INTERNAL_ERROR, NULL);
     }
 
@@ -135,6 +135,69 @@ bool flecs_query_with(
     return true;
 }
 
+static
+bool flecs_query_all(
+    const ecs_query_op_t *op,
+    bool redo,
+    const ecs_query_run_ctx_t *ctx)
+{
+    int8_t field_index = op->field_index;
+    ecs_iter_t *it = ctx->it;
+
+    uint64_t written = ctx->written[ctx->op_index];
+    if (written & (1ull << op->src.var)) {
+        if (field_index != -1) {
+            it->ids[field_index] = EcsWildcard;
+        }
+        return !redo;
+    } else {
+        ecs_query_all_ctx_t *op_ctx = flecs_op_ctx(ctx, all);
+        ecs_world_t *world = ctx->world;
+        ecs_sparse_t *tables = &world->store.tables;
+        bool match_empty = ctx->query->pub.flags & EcsQueryMatchEmptyTables;
+        ecs_table_t *table;
+
+        if (!redo) {
+            op_ctx->cur = 0;
+            op_ctx->dummy_tr.column = -1;
+            op_ctx->dummy_tr.index = -1;
+            op_ctx->dummy_tr.count = 0;
+            op_ctx->dummy_tr.hdr.cache = NULL;
+            if (field_index != -1) {
+                it->ids[field_index] = EcsWildcard;
+                it->trs[field_index] = &op_ctx->dummy_tr;
+            }
+            table = &world->store.root;
+        } else if (op_ctx->cur < flecs_sparse_count(tables)) {
+            table = flecs_sparse_get_dense_t(
+                tables, ecs_table_t, op_ctx->cur);
+        } else {
+            return false;
+        }
+
+repeat:
+        op_ctx->cur ++;
+
+        if (match_empty || ecs_table_count(table)) {
+            if (!flecs_query_table_filter(table, op->other, 
+                (EcsTableNotQueryable|EcsTableIsPrefab|EcsTableIsDisabled)))
+            {
+                op_ctx->dummy_tr.hdr.table = table;
+                flecs_query_var_set_range(op, op->src.var, table, 0, 0, ctx);
+                return true;
+            }
+        }
+
+        if (op_ctx->cur < flecs_sparse_count(tables)) {
+            table = flecs_sparse_get_dense_t(
+                tables, ecs_table_t, op_ctx->cur);
+            goto repeat;
+        }
+
+        return false;
+    }
+}
+
 static
 bool flecs_query_and(
     const ecs_query_op_t *op,
@@ -162,20 +225,20 @@ bool flecs_query_select_id(
 
     if (!redo) {
         ecs_id_t id = it->ids[field];
-        ecs_id_record_t *idr = op_ctx->idr;
-        if (!idr || idr->id != id) {
-            idr = op_ctx->idr = flecs_id_record_get(ctx->world, id);
-            if (!idr) {
+        ecs_component_record_t *cdr = op_ctx->cdr;
+        if (!cdr || cdr->id != id) {
+            cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
+            if (!cdr) {
                 return false;
             }
         }
 
         if (ctx->query->pub.flags & EcsQueryMatchEmptyTables) {
-            if (!flecs_table_cache_all_iter(&idr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_all_iter(&cdr->cache, &op_ctx->it)) {
                 return false;
             }
         } else {
-            if (!flecs_table_cache_iter(&idr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_iter(&cdr->cache, &op_ctx->it)) {
                 return false;
             }
         }
@@ -213,20 +276,18 @@ bool flecs_query_with_id(
     ecs_assert(field != -1, ECS_INTERNAL_ERROR, NULL);
 
     ecs_table_t *table = flecs_query_get_table(op, &op->src, EcsQuerySrc, ctx);
-    if (!table) {
-        return false;
-    }
+    ecs_assert(table != NULL, ECS_INTERNAL_ERROR, NULL);
 
     ecs_id_t id = it->ids[field];
-    ecs_id_record_t *idr = op_ctx->idr;
-    if (!idr || idr->id != id) {
-        idr = op_ctx->idr = flecs_id_record_get(ctx->world, id);
-        if (!idr) {
+    ecs_component_record_t *cdr = op_ctx->cdr;
+    if (!cdr || cdr->id != id) {
+        cdr = op_ctx->cdr = flecs_components_get(ctx->world, id);
+        if (!cdr) {
             return false;
         }
     }
 
-    ecs_table_record_t *tr = flecs_id_record_get_table(idr, table);
+    const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
     if (!tr) {
         return false;
     }
@@ -299,26 +360,11 @@ bool flecs_query_and_any(
         ctx->it->ids[field] = flecs_query_op_get_id(op, ctx);
     }
 
-    ctx->it->trs[field] = (ecs_table_record_t*)op_ctx->it.cur;
+    ctx->it->trs[field] = (const ecs_table_record_t*)op_ctx->it.cur;
 
     return result;
 }
 
-static
-bool flecs_query_only_any(
-    const ecs_query_op_t *op,
-    bool redo,
-    const ecs_query_run_ctx_t *ctx)
-{
-    uint64_t written = ctx->written[ctx->op_index];
-    if (flecs_ref_is_written(op, &op->src, EcsQuerySrc, written)) {
-        return flecs_query_and_any(op, redo, ctx);
-    } else {
-        return flecs_query_select_w_id(op, redo, ctx, EcsAny, 
-            (EcsTableNotQueryable|EcsTableIsPrefab|EcsTableIsDisabled));
-    }
-}
-
 static
 bool flecs_query_triv(
     const ecs_query_op_t *op,
@@ -380,8 +426,8 @@ int32_t flecs_query_next_inheritable_id(
 {
     int32_t i;
     for (i = index; i < type->count; i ++) {
-        ecs_id_record_t *idr = flecs_id_record_get(world, type->array[i]);
-        if (!(idr->flags & EcsIdOnInstantiateDontInherit)) {
+        ecs_component_record_t *cdr = flecs_components_get(world, type->array[i]);
+        if (!(cdr->flags & EcsIdOnInstantiateDontInherit)) {
             return i;
         }
     }
@@ -407,11 +453,9 @@ bool flecs_query_x_from(
         op_ctx->type_id = type_id;
         ecs_assert(ecs_is_alive(world, type_id), ECS_INTERNAL_ERROR, NULL);
         ecs_record_t *r = flecs_entities_get(world, type_id);
-        ecs_table_t *table;
-        if (!r || !(table = r->table)) {
-            /* Nothing to match */
-            return false;
-        }
+        ecs_assert(r != NULL, ECS_INTERNAL_ERROR, NULL);
+        ecs_table_t *table = r->table;
+        ecs_assert(table != NULL, ECS_INTERNAL_ERROR, NULL);
 
         /* Find first id to test against. Skip ids with DontInherit flag. */
         type = op_ctx->type = &table->type;
@@ -477,16 +521,16 @@ bool flecs_query_x_from(
              * components from the type list */
             if (op_ctx->cur_id_index != op_ctx->first_id_index) {
                 for (i = op_ctx->first_id_index; i < op_ctx->cur_id_index; i ++) {
-                    ecs_id_record_t *idr = flecs_id_record_get(world, ids[i]);
-                    if (!idr) {
+                    ecs_component_record_t *cdr = flecs_components_get(world, ids[i]);
+                    if (!cdr) {
                         continue;
                     }
 
-                    if (idr->flags & EcsIdOnInstantiateDontInherit) {
+                    if (cdr->flags & EcsIdOnInstantiateDontInherit) {
                         continue;
                     }
                     
-                    if (flecs_id_record_get_table(idr, src_table) != NULL) {
+                    if (flecs_component_get_table(cdr, src_table) != NULL) {
                         /* Already matched */
                         break;
                     }
@@ -500,8 +544,8 @@ bool flecs_query_x_from(
 
         if (oper == EcsAndFrom || oper == EcsNotFrom || src_written) {
             for (i = id_index; i < type->count; i ++) {
-                ecs_id_record_t *idr = flecs_id_record_get(world, ids[i]);
-                if (!idr) {
+                ecs_component_record_t *cdr = flecs_components_get(world, ids[i]);
+                if (!cdr) {
                     if (oper == EcsAndFrom) {
                         return false;
                     } else {
@@ -509,11 +553,11 @@ bool flecs_query_x_from(
                     }
                 }
 
-                if (idr->flags & EcsIdOnInstantiateDontInherit) {
+                if (cdr->flags & EcsIdOnInstantiateDontInherit) {
                     continue;
                 }
 
-                if (flecs_id_record_get_table(idr, src_table) == NULL) {
+                if (flecs_component_get_table(cdr, src_table) == NULL) {
                     if (oper == EcsAndFrom) {
                         break; /* Must have all ids */
                     }
@@ -576,11 +620,11 @@ bool flecs_query_ids(
         return false;
     }
 
-    ecs_id_record_t *cur;
+    ecs_component_record_t *cur;
     ecs_id_t id = flecs_query_op_get_id(op, ctx);
 
     {
-        cur = flecs_id_record_get(ctx->world, id);
+        cur = flecs_components_get(ctx->world, id);
         if (!cur || !cur->cache.tables.count) {
             return false;
         }
@@ -605,7 +649,7 @@ bool flecs_query_idsright(
     const ecs_query_run_ctx_t *ctx)
 {
     ecs_query_ids_ctx_t *op_ctx = flecs_op_ctx(ctx, ids);
-    ecs_id_record_t *cur;
+    ecs_component_record_t *cur;
 
     if (!redo) {
         ecs_id_t id = flecs_query_op_get_id(op, ctx);
@@ -617,7 +661,7 @@ bool flecs_query_idsright(
             return true;
         }
 
-        cur = op_ctx->cur = flecs_id_record_get(ctx->world, id);
+        cur = op_ctx->cur = flecs_components_get(ctx->world, id);
         if (!cur) {
             return false;
         }
@@ -627,14 +671,21 @@ bool flecs_query_idsright(
         }
     }
 
+next:
     do {
-        cur = op_ctx->cur = op_ctx->cur->first.next;
+        cur = op_ctx->cur = flecs_component_first_next(op_ctx->cur);
     } while (cur && !cur->cache.tables.count); /* Skip empty ids */
 
     if (!cur) {
         return false;
     }
 
+    if (cur->id == ecs_pair(EcsChildOf, 0)) {
+        /* Skip the special (ChildOf, 0) entry for root entities, as 0 is
+         * not a valid target and could be matched by (ChildOf, *) */
+        goto next;
+    }
+
     flecs_query_set_vars(op, cur->id, ctx);
 
     if (op->field_index != -1) {
@@ -655,7 +706,7 @@ bool flecs_query_idsleft(
     const ecs_query_run_ctx_t *ctx)
 {
     ecs_query_ids_ctx_t *op_ctx = flecs_op_ctx(ctx, ids);
-    ecs_id_record_t *cur;
+    ecs_component_record_t *cur;
 
     if (!redo) {
         ecs_id_t id = flecs_query_op_get_id(op, ctx);
@@ -667,7 +718,7 @@ bool flecs_query_idsleft(
             return true;
         }
 
-        cur = op_ctx->cur = flecs_id_record_get(ctx->world, id);
+        cur = op_ctx->cur = flecs_components_get(ctx->world, id);
         if (!cur) {
             return false;
         }
@@ -678,7 +729,7 @@ bool flecs_query_idsleft(
     }
 
     do {
-        cur = op_ctx->cur = op_ctx->cur->second.next;
+        cur = op_ctx->cur = flecs_component_second_next(op_ctx->cur);
     } while (cur && !cur->cache.tables.count); /* Skip empty ids */ 
 
     if (!cur) {
@@ -709,9 +760,7 @@ bool flecs_query_each(
 
     ecs_table_range_t range = flecs_query_var_get_range(op->first.var, ctx);
     ecs_table_t *table = range.table;
-    if (!table) {
-        return false;
-    }
+    ecs_assert(table != NULL, ECS_INTERNAL_ERROR, NULL);
 
     if (!redo) {
         if (!ecs_table_count(table)) {
@@ -1340,12 +1389,12 @@ bool flecs_query_dispatch(
     ecs_query_run_ctx_t *ctx)
 {
     switch(op->kind) {
+    case EcsQueryAll: return flecs_query_all(op, redo, ctx);
     case EcsQueryAnd: return flecs_query_and(op, redo, ctx);
     case EcsQueryAndAny: return flecs_query_and_any(op, redo, ctx);
     case EcsQueryTriv: return flecs_query_triv(op, redo, ctx);
     case EcsQueryCache: return flecs_query_cache(op, redo, ctx);
     case EcsQueryIsCache: return flecs_query_is_cache(op, redo, ctx);
-    case EcsQueryOnlyAny: return flecs_query_only_any(op, redo, ctx);
     case EcsQueryUp: return flecs_query_up(op, redo, ctx);
     case EcsQuerySelfUp: return flecs_query_self_up(op, redo, ctx);
     case EcsQueryWith: return flecs_query_with(op, redo, ctx);
