@@ -27,17 +27,17 @@ bool flecs_query_trivial_search_init(
         ecs_assert(t != query->term_count, ECS_INTERNAL_ERROR, NULL);
         op_ctx->start_from = t;
 
-        ecs_id_record_t *idr = flecs_id_record_get(ctx->world, query->ids[t]);
-        if (!idr) {
+        ecs_component_record_t *cdr = flecs_components_get(ctx->world, query->ids[t]);
+        if (!cdr) {
             return false;
         }
 
         if (query->flags & EcsQueryMatchEmptyTables) {
-            if (!flecs_table_cache_all_iter(&idr->cache, &op_ctx->it)){
+            if (!flecs_table_cache_all_iter(&cdr->cache, &op_ctx->it)){
                 return false;
             }
         } else {
-            if (!flecs_table_cache_iter(&idr->cache, &op_ctx->it)) {
+            if (!flecs_table_cache_iter(&cdr->cache, &op_ctx->it)) {
                 return false;
             }
         }
@@ -90,13 +90,13 @@ bool flecs_query_trivial_search(
             }
 
             const ecs_term_t *term = &terms[t];
-            ecs_id_record_t *idr = flecs_id_record_get(ctx->world, term->id);
-            if (!idr) {
+            ecs_component_record_t *cdr = flecs_components_get(ctx->world, term->id);
+            if (!cdr) {
                 break;
             }
 
-            const ecs_table_record_t *tr_with = flecs_id_record_get_table(
-                idr, table);
+            const ecs_table_record_t *tr_with = flecs_component_get_table(
+                cdr, table);
             if (!tr_with) {
                 break;
             }
@@ -145,13 +145,13 @@ next:
         }
 
         for (t = 1; t < term_count; t ++) {
-            ecs_id_record_t *idr = flecs_id_record_get(ctx->world, ids[t]);
-            if (!idr) {
+            ecs_component_record_t *cdr = flecs_components_get(ctx->world, ids[t]);
+            if (!cdr) {
                 return false;
             }
 
-            const ecs_table_record_t *tr_with = flecs_id_record_get_table(
-                idr, table);
+            const ecs_table_record_t *tr_with = flecs_component_get_table(
+                cdr, table);
             if (!tr_with) {
                 goto next;
             }
@@ -192,12 +192,12 @@ bool flecs_query_trivial_test(
             }
 
             const ecs_term_t *term = &terms[t];
-            ecs_id_record_t *idr = flecs_id_record_get(q->world, term->id);
-            if (!idr) {
+            ecs_component_record_t *cdr = flecs_components_get(q->world, term->id);
+            if (!cdr) {
                 return false;
             }
 
-            const ecs_table_record_t *tr = flecs_id_record_get_table(idr, table);
+            const ecs_table_record_t *tr = flecs_component_get_table(cdr, table);
             if (!tr) {
                 return false;
             }
