@@ -42,9 +42,9 @@ extern char *flecs_this_name_array;
 
 /* -- Instruction kinds -- */
 typedef enum {
+    EcsQueryAll,            /* Yield all tables */
     EcsQueryAnd,            /* And operator: find or match id against variable source */
     EcsQueryAndAny,         /* And operator with support for matching Any src/id */
-    EcsQueryOnlyAny,        /* Dedicated instruction for _ queries where the src is unknown */
     EcsQueryTriv,           /* Trivial search (batches multiple terms) */
     EcsQueryCache,          /* Cached search */
     EcsQueryIsCache,        /* Cached search for queries that are entirely cached */
@@ -126,9 +126,15 @@ typedef struct ecs_query_op_t {
     ecs_flags64_t written;     /* Bitset with variables written by op */
 } ecs_query_op_t;
 
- /* And context */
+/* All context */
 typedef struct {
-    ecs_id_record_t *idr;
+    int32_t cur;
+    ecs_table_record_t dummy_tr;
+} ecs_query_all_ctx_t;
+
+/* And context */
+typedef struct {
+    ecs_component_record_t *cdr;
     ecs_table_cache_iter_t it;
     int16_t column;
     int16_t remaining;
@@ -136,7 +142,7 @@ typedef struct {
 
 /* Union context */
 typedef struct {
-    ecs_id_record_t *idr;
+    ecs_component_record_t *cdr;
     ecs_table_range_t range;
     ecs_map_iter_t tgt_iter;
     ecs_entity_t cur;
@@ -185,8 +191,8 @@ typedef struct {
     ecs_entity_t trav;
     ecs_id_t with;
     ecs_id_t matched;
-    ecs_id_record_t *idr_with;
-    ecs_id_record_t *idr_trav;
+    ecs_component_record_t *idr_with;
+    ecs_component_record_t *idr_trav;
     ecs_trav_down_t *down;
     int32_t cache_elem;
     ecs_trav_up_cache_t cache;
@@ -196,13 +202,13 @@ typedef struct {
  * traversal iterates and caches the entire tree. */
 typedef struct {
     ecs_entity_t entity;
-    ecs_id_record_t *idr;
+    ecs_component_record_t *cdr;
     const ecs_table_record_t *tr;
 } ecs_trav_elem_t;
 
 typedef struct {
     ecs_id_t id;
-    ecs_id_record_t *idr;
+    ecs_component_record_t *cdr;
     ecs_vec_t entities;
     bool up;
 } ecs_trav_cache_t;
@@ -237,7 +243,7 @@ typedef struct {
 
 /* Ids context */
 typedef struct {
-    ecs_id_record_t *cur;
+    ecs_component_record_t *cur;
 } ecs_query_ids_ctx_t;
 
 /* Control flow context */
@@ -283,6 +289,7 @@ typedef struct {
 
 typedef struct ecs_query_op_ctx_t {
     union {
+        ecs_query_all_ctx_t all;
         ecs_query_and_ctx_t and;
         ecs_query_xfrom_ctx_t xfrom;
         ecs_query_up_ctx_t up;
@@ -362,6 +369,10 @@ struct ecs_query_impl_t {
     char *tokens;                 /* Buffer with string tokens used by terms */
     int32_t *monitor;             /* Change monitor for fields with fixed src */
 
+#ifdef FLECS_DEBUG
+    ecs_termset_t final_terms;    /* Terms that don't use component inheritance */
+#endif
+
     /* Query cache */
     struct ecs_query_cache_t *cache; /* Cache, if query contains cached terms */
 
@@ -389,6 +400,7 @@ struct ecs_query_cache_table_match_t {
     const ecs_table_record_t **trs;  /* Information about where to find field in table */
     ecs_id_t *ids;                   /* Resolved (component) ids for current table */
     ecs_entity_t *sources;           /* Subjects (sources) of ids */
+    ecs_table_t **tables;            /* Tables for fields with non-$this source */
     ecs_termset_t set_fields;        /* Fields that are set */
     ecs_termset_t up_fields;         /* Fields that are matched through traversal */
     uint64_t group_id;               /* Value used to organize tables in groups */
@@ -400,7 +412,7 @@ struct ecs_query_cache_table_match_t {
 
 /** Table record type for query table cache. A query only has one per table. */
 typedef struct ecs_query_cache_table_t {
-    ecs_table_cache_hdr_t hdr;       /* Header for ecs_table_cache_t */
+    const ecs_table_cache_hdr_t hdr;       /* Header for ecs_table_cache_t */
     ecs_query_cache_table_match_t *first;  /* List with matches for table */
     ecs_query_cache_table_match_t *last;   /* Last discovered match for table */
     uint64_t table_id;
@@ -428,9 +440,8 @@ typedef struct ecs_query_cache_event_t {
 
 /* Query level block allocators have sizes that depend on query field count */
 typedef struct ecs_query_cache_allocators_t {
-    ecs_block_allocator_t trs;
+    ecs_block_allocator_t pointers;
     ecs_block_allocator_t ids;
-    ecs_block_allocator_t sources;
     ecs_block_allocator_t monitors;
 } ecs_query_cache_allocators_t;
 
