@@ -42,6 +42,7 @@ ecs_cmd_t* flecs_cmd_new_batched(
 
     int32_t cur = ecs_vec_count(cmds);
     ecs_cmd_t *cmd = flecs_cmd_new(stage);
+    bool is_new = false;
     if (entry) {
         if (entry->first == -1) {
             /* Existing but invalidated entry */
@@ -50,16 +51,25 @@ ecs_cmd_t* flecs_cmd_new_batched(
         } else {
             int32_t last = entry->last;
             ecs_cmd_t *arr = ecs_vec_first_t(cmds, ecs_cmd_t);
-            ecs_assert(arr[last].entity == e, ECS_INTERNAL_ERROR, NULL);
-            ecs_cmd_t *last_op = &arr[last];
-            last_op->next_for_entity = cur;
-            if (last == entry->first) {
-                /* Flip sign bit so flush logic can tell which command
-                 * is the first for an entity */
-                last_op->next_for_entity *= -1;
+            if (arr[last].entity == e) {
+                ecs_cmd_t *last_op = &arr[last];
+                last_op->next_for_entity = cur;
+                if (last == entry->first) {
+                    /* Flip sign bit so flush logic can tell which command
+                    * is the first for an entity */
+                    last_op->next_for_entity *= -1;
+                }
+            } else {
+                /* Entity with different version was in the same queue. Discard
+                 * the old entry and create a new one. */
+                is_new = true;
             }
         }
     } else {
+        is_new = true;
+    }
+
+    if (is_new) {
         cmd->entry = entry = flecs_sparse_ensure_fast_t(
             &stage->cmd->entries, ecs_cmd_entry_t, e);
         entry->first = cur;
@@ -144,13 +154,28 @@ bool flecs_defer_cmd(
     return false;
 }
 
+bool flecs_defer_new(
+    ecs_stage_t *stage,
+    ecs_entity_t entity)
+{
+    if (flecs_defer_cmd(stage)) {
+        ecs_cmd_t *cmd = flecs_cmd_new(stage);
+        if (cmd) {
+            cmd->kind = EcsCmdNew;
+            cmd->entity = entity;
+        }
+        return true;
+    }
+    return false; 
+}
+
 bool flecs_defer_modified(
     ecs_stage_t *stage,
     ecs_entity_t entity,
     ecs_id_t id)
 {
     if (flecs_defer_cmd(stage)) {
-        ecs_cmd_t *cmd = flecs_cmd_new_batched(stage, entity);
+        ecs_cmd_t *cmd = flecs_cmd_new(stage);
         if (cmd) {
             cmd->kind = EcsCmdModified;
             cmd->id = id;
@@ -328,26 +353,26 @@ void* flecs_defer_set(
 
     /* Find type info for id */
     const ecs_type_info_t *ti = NULL;
-    ecs_id_record_t *idr = flecs_id_record_get(world, id);
-    if (!idr) {
-        /* If idr doesn't exist yet, create it but only if the 
+    ecs_component_record_t *cdr = flecs_components_get(world, id);
+    if (!cdr) {
+        /* If cdr doesn't exist yet, create it but only if the 
          * application is not multithreaded. */
         if (world->flags & EcsWorldMultiThreaded) {
             ti = ecs_get_type_info(world, id);
         } else {
             /* When not in multi threaded mode, it's safe to find or 
-             * create the id record. */
-            idr = flecs_id_record_ensure(world, id);
-            ecs_assert(idr != NULL, ECS_INTERNAL_ERROR, NULL);
+             * create the component record. */
+            cdr = flecs_components_ensure(world, id);
+            ecs_assert(cdr != NULL, ECS_INTERNAL_ERROR, NULL);
             
-            /* Get type_info from id record. We could have called 
+            /* Get type_info from component record. We could have called 
              * ecs_get_type_info directly, but since this function can be
-             * expensive for pairs, creating the id record ensures we can
+             * expensive for pairs, creating the component record ensures we can
              * find the type_info quickly for subsequent operations. */
-            ti = idr->type_info;
+            ti = cdr->type_info;
         }
     } else {
-        ti = idr->type_info;
+        ti = cdr->type_info;
     }
 
     /* If the id isn't associated with a type, we can't set anything */
@@ -363,13 +388,13 @@ void* flecs_defer_set(
      * component of a prefab. */
     void *existing = NULL;
     ecs_table_t *table = NULL;
-    if (idr) {
-        /* Entity can only have existing component if id record exists */
+    if (cdr) {
+        /* Entity can only have existing component if component record exists */
         ecs_record_t *r = flecs_entities_get(world, entity);
         table = r->table;
         if (r && table) {
-            const ecs_table_record_t *tr = flecs_id_record_get_table(
-                idr, table);
+            const ecs_table_record_t *tr = flecs_component_get_table(
+                cdr, table);
             if (tr) {
                 if (tr->column != -1) {
                     /* Entity has the component */
@@ -377,9 +402,9 @@ void* flecs_defer_set(
                     existing = ECS_ELEM(existing, size, 
                         ECS_RECORD_TO_ROW(r->row));
                 } else {
-                    ecs_assert(idr->flags & EcsIdIsSparse, 
+                    ecs_assert(cdr->flags & EcsIdIsSparse, 
                         ECS_NOT_A_COMPONENT, NULL);
-                    existing = flecs_sparse_get_any(idr->sparse, 0, entity);
+                    existing = flecs_sparse_get_any(cdr->sparse, 0, entity);
                 }
             }
         }
@@ -423,7 +448,7 @@ void* flecs_defer_set(
             /* Check if entity inherits component */
             void *base = NULL;
             if (table && (table->flags & EcsTableHasIsA)) {
-                base = flecs_get_base_component(world, table, id, idr, 0);
+                base = flecs_get_base_component(world, table, id, cdr, 0);
             }
 
             if (!base) {
@@ -469,7 +494,7 @@ void* flecs_defer_set(
         /* If component didn't exist yet, insert command that will create it */
         cmd->kind = cmd_kind;
         cmd->id = id;
-        cmd->idr = idr;
+        cmd->cdr = cdr;
         cmd->entity = entity;
         cmd->is._1.size = size;
         cmd->is._1.value = cmd_value;
