@@ -8,5 +8,5 @@ void Allocator_init_fini_empty(void) {
     ecs_allocator_t a;
     flecs_allocator_init(&a);
     flecs_allocator_fini(&a);
-    test_assert(true); // make sure there are no leaks, crashses
+    test_assert(true); // make sure there are no leaks, crashes
 }
