@@ -2280,3 +2280,573 @@ void ComponentLifecycle_set_multiple_hooks(void) {
     ecs.release(); /* destroys world */
     test_int(removes, 2); /* two instances of `Pod` removed */
 }
+
+struct WithGreaterThan {
+    int value;
+    bool operator>(const WithGreaterThan &other) const {
+        return value > other.value;
+    }
+};
+
+struct WithLessThan {
+    int value;
+    bool operator<(const WithLessThan &other) const {
+        return value < other.value;
+    }
+};
+
+struct WithLessAndGreaterThan {
+    int value;
+    bool operator<(const WithLessAndGreaterThan &other) const {
+        return value < other.value;
+    }
+    bool operator>(const WithLessAndGreaterThan &other) const {
+        return value > other.value;
+    }
+};
+
+
+struct WithEqualsAndGreaterThan {
+    int value;
+    bool operator==(const WithEqualsAndGreaterThan &other) const {
+        return value == other.value;
+    }
+    bool operator>(const WithEqualsAndGreaterThan &other) const {
+        return value > other.value;
+    }
+};
+
+struct WithEqualsAndLessThan {
+    int value;
+    bool operator==(const WithEqualsAndLessThan &other) const {
+        return value == other.value;
+    }
+    bool operator<(const WithEqualsAndLessThan &other) const {
+        return value < other.value;
+    }
+};
+
+struct WithEqualsOnly {
+    int value;
+    bool operator==(const WithEqualsOnly &other) const {
+        return value == other.value;
+    }
+};
+
+struct WithoutOperators {
+    int16_t value;
+};
+
+static
+int compare(flecs::world& ecs, flecs::entity_t id, const void *a, const void *b) {
+    const ecs_type_info_t* ti = ecs_get_type_info(ecs, id);
+    return ti->hooks.cmp(a, b, ti);
+}
+
+static
+bool equals(flecs::world& ecs, flecs::entity_t id, const void *a, const void *b) {
+    const ecs_type_info_t* ti = ecs_get_type_info(ecs, id);
+    return ti->hooks.equals(a, b, ti);
+}
+
+void ComponentLifecycle_compare_WithGreaterThan(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithGreaterThan>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* generate cmp operator from C++ operator> */
+    component.on_compare(); 
+
+    hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    /* `equals` is automatically generated: */
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    WithGreaterThan a = {1};
+    WithGreaterThan b = {2};
+    WithGreaterThan c = {1};
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    /* test using autogenerated equals operator: */
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);     
+}
+
+void ComponentLifecycle_compare_WithLessThan(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithLessThan>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* generate cmp operator from C++ operator< */
+    component.on_compare(); 
+
+    hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    /* `equals` is automatically generated: */
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+
+    WithLessThan a = {1};
+    WithLessThan b = {2};
+    WithLessThan c = {1};
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    /* test using autogenerated equals operator: */
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);         
+}
+
+void ComponentLifecycle_compare_WithLessAndGreaterThan(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithLessAndGreaterThan>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* generate cmp operator from C++ operator> and operator< */
+    component.on_compare(); 
+
+    hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    /* `equals` is automatically generated: */
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    WithLessAndGreaterThan a = {1};
+    WithLessAndGreaterThan b = {2};
+    WithLessAndGreaterThan c = {1};
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    /* test using autogenerated equals operator: */
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);     
+}
+
+void ComponentLifecycle_compare_WithEqualsAndGreaterThan(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithEqualsAndGreaterThan>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* generate cmp operator from C++ operator> and operator== */
+    component.on_compare(); 
+
+    hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    /* `equals` is automatically generated: */
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    WithEqualsAndGreaterThan a = {1};
+    WithEqualsAndGreaterThan b = {2};
+    WithEqualsAndGreaterThan c = {1};
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    /* test using equals operator: */
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);
+}
+
+void ComponentLifecycle_compare_WithEqualsAndLessThan(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithEqualsAndLessThan>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* generate cmp operator from C++ operator< and operator== */
+    component.on_compare(); 
+
+    hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    /* `equals` is automatically generated: */
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+
+    WithEqualsAndLessThan a = {1};
+    WithEqualsAndLessThan b = {2};
+    WithEqualsAndLessThan c = {1};
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    /* test using equals operator: */
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);
+
+
+}
+
+void ComponentLifecycle_compare_WithEqualsOnly(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithEqualsOnly>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* generate equals operator from C++ operator== */
+    component.on_equals(); 
+
+    hooks = ecs_get_hooks_id(ecs, component);
+
+    /* can't compare if no < or > operators are defined */
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+
+    /* operator equals is defined: */
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    WithEqualsOnly a = {1};
+    WithEqualsOnly b = {2};
+    WithEqualsOnly c = {1};
+    
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);
+
+}
+
+void ComponentLifecycle_compare_WithoutOperators(void) {
+    flecs::world ecs;
+
+    auto component = ecs.component<WithoutOperators>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    /* can't compare if no operators are defined at all */
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL);
+
+    /* define operator equals via a explicit callback: */
+    component.on_equals([](
+        const WithoutOperators *a, 
+        const WithoutOperators *b, 
+        const ecs_type_info_t* ti) -> bool {
+            return a->value == b->value;
+        });
+
+    hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL);
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    WithoutOperators a = {1};
+    WithoutOperators b = {2};
+    WithoutOperators c = {1};
+    
+    test_assert(equals(ecs, component, &a, &b) == false);
+    test_assert(equals(ecs, component, &a, &c) == true);
+    test_assert(equals(ecs, component, &a, &a) == true);    
+
+    component.on_compare([](
+        const WithoutOperators *a, 
+        const WithoutOperators *b, 
+        const ecs_type_info_t *ti) -> int {
+            return a->value - b->value;
+        });
+
+    hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);    
+}
+
+
+template <typename T>
+struct TestUnsignedEnum {
+    enum Type : T {
+        Red = 1,
+        Yellow = 2,
+        Blue = 3
+    };
+};
+
+template <typename T>
+struct TestSignedEnum {
+    enum Type : T {
+        Red = -1,
+        Yellow = 0,
+        Blue = 1
+    };
+};
+
+void ComponentLifecycle_compare_uint8_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum8 = TestUnsignedEnum<uint8_t>::Type;
+
+    auto component = ecs.component<Enum8>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum8 a = Enum8::Red;
+    Enum8 b = Enum8::Yellow;
+    Enum8 c = Enum8::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+
+void ComponentLifecycle_compare_uint16_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum16 = TestUnsignedEnum<uint16_t>::Type;
+
+    auto component = ecs.component<Enum16>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum16 a = Enum16::Red;
+    Enum16 b = Enum16::Yellow;
+    Enum16 c = Enum16::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);    
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+void ComponentLifecycle_compare_uint32_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum32 = TestUnsignedEnum<uint32_t>::Type;
+
+    auto component = ecs.component<Enum32>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum32 a = Enum32::Red;
+    Enum32 b = Enum32::Yellow;
+    Enum32 c = Enum32::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);    
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+
+void ComponentLifecycle_compare_uint64_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum64 = TestUnsignedEnum<uint64_t>::Type;
+
+    auto component = ecs.component<Enum64>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum64 a = Enum64::Red;
+    Enum64 b = Enum64::Yellow;
+    Enum64 c = Enum64::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);    
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+void ComponentLifecycle_compare_int8_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum8 = TestSignedEnum<int8_t>::Type;
+
+    auto component = ecs.component<Enum8>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum8 a = Enum8::Red;
+    Enum8 b = Enum8::Yellow;
+    Enum8 c = Enum8::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+void ComponentLifecycle_compare_int16_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum16 = TestSignedEnum<int16_t>::Type;
+
+    auto component = ecs.component<Enum16>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum16 a = Enum16::Red;
+    Enum16 b = Enum16::Yellow;
+    Enum16 c = Enum16::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+void ComponentLifecycle_compare_int32_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum32 = TestSignedEnum<int32_t>::Type;
+
+    auto component = ecs.component<Enum32>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum32 a = Enum32::Red;
+    Enum32 b = Enum32::Yellow;
+    Enum32 c = Enum32::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
+
+void ComponentLifecycle_compare_int64_Enum(void) {
+
+    flecs::world ecs;
+
+    using Enum64 = TestSignedEnum<int64_t>::Type;
+
+    auto component = ecs.component<Enum64>();
+
+    const ecs_type_hooks_t* hooks = ecs_get_hooks_id(ecs, component);
+
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_CMP_ILLEGAL));
+    test_assert(!(hooks->flags & ECS_TYPE_HOOK_EQUALS_ILLEGAL));
+
+    Enum64 a = Enum64::Red;
+    Enum64 b = Enum64::Yellow;
+    Enum64 c = Enum64::Red;
+
+    test_assert(compare(ecs, component, &a, &b) < 0);
+    test_assert(compare(ecs, component, &b, &a) > 0);
+    test_assert(compare(ecs, component, &a, &c) == 0);
+    test_assert(compare(ecs, component, &b, &c) > 0);
+    test_assert(compare(ecs, component, &c, &b) < 0);
+    test_assert(compare(ecs, component, &b, &b) == 0);
+
+    test_assert(equals(ecs, component, &a, &c));
+    test_assert(equals(ecs, component, &b, &b));
+}
