@@ -494,6 +494,15 @@ void Enum_add_enum_constant(void) {
     test_assert(id == ecs.pair(r, c));
 }
 
+void Enum_add_enum_constant_explicit(void) {
+    flecs::world ecs;
+
+    auto e = ecs.entity().add<StandardEnum>(StandardEnum::Red);
+    test_assert(e.has<StandardEnum>(StandardEnum::Red));
+    test_assert(!e.has<StandardEnum>(StandardEnum::Green));
+    test_assert(!e.has<StandardEnum>(StandardEnum::Blue));
+}
+
 void Enum_add_enum_class_constant(void) {
     flecs::world ecs;
 
@@ -588,7 +597,7 @@ void Enum_remove_enum(void) {
     auto e = ecs.entity().add(StandardEnum::Green);
     test_assert(e.has(StandardEnum::Green));
 
-    e.remove<StandardEnum>();
+    e.remove<StandardEnum>(flecs::Wildcard);
     test_assert(!e.has(StandardEnum::Green));
 }
 
@@ -1580,3 +1589,67 @@ void Enum_enum_w_one_constant_index_of(void) {
     auto one_type = flecs::enum_type<OneConstant>(ecs);
     test_int(one_type.index_by_value(0), 0);
 }
+
+void Enum_runtime_type_constant_u8_template(void) {
+    flecs::world ecs;
+
+    auto comp = ecs.component("TestEnumConstant");
+    comp.set<flecs::Component>({ sizeof(uint8_t), alignof(uint8_t) });
+    comp.set<flecs::Enum>({ flecs::U8 });
+
+    comp.constant<uint8_t>("First", 1)
+        .constant<uint8_t>("Second", 2)
+        .constant<uint8_t>("Third", 3);
+
+    auto first = comp.lookup("First");
+    auto second = comp.lookup("Second");
+    auto third = comp.lookup("Third");
+
+    test_assert(first.is_valid());
+    test_assert(second.is_valid());
+    test_assert(third.is_valid());
+
+    const uint8_t *val_first = first.get_second<uint8_t>(flecs::Constant);
+    const uint8_t *val_second = second.get_second<uint8_t>(flecs::Constant);
+    const uint8_t *val_third = third.get_second<uint8_t>(flecs::Constant);
+
+    test_true(val_first != nullptr && *val_first == 1);
+    test_true(val_second != nullptr && *val_second == 2);
+    test_true(val_third != nullptr && *val_third == 3);
+}
+
+enum ThreadTest {
+    A, B, C, D, E, F, G, H, I, J
+};
+
+void* thread_cb(void* arg) {
+    flecs::world ecs;
+
+    ecs.component<ThreadTest>();
+
+    flecs::entity e = ecs.entity().add(ThreadTest::B);
+    test_assert(e.has(ThreadTest::B));
+
+    e.add(ThreadTest::J);
+    test_assert(!e.has(ThreadTest::B));
+    test_assert(e.has(ThreadTest::J));
+
+    return NULL;
+}
+
+void Enum_multithreaded_enum_registration(void) {
+    ecs_os_init();
+
+    ecs_os_thread_t *threads = ecs_os_malloc_n(ecs_os_thread_t, 100);
+
+    for (int i = 0; i < 100; i ++) {
+        threads[i] = ecs_os_thread_new(thread_cb, NULL);
+        test_assert(threads[i]);
+    }
+
+    for (int i = 0; i < 100; i ++) {
+        ecs_os_thread_join(threads[i]);
+    }
+
+    ecs_os_free(threads);
+}
