@@ -191,6 +191,24 @@ void Refs_try_get(void) {
     test_assert(p.try_get() == nullptr);
 }
 
+void Refs_try_get_after_delete(void) {
+    flecs::world world;
+
+    flecs::entity e = world.entity().set(Position{10, 20});
+
+    flecs::ref<Position> p = e.get_ref<Position>();
+    Position *ptr = p.try_get();
+    test_assert(ptr != nullptr);
+    test_int(ptr->x, 10);
+    test_int(ptr->y, 20);
+
+    e.destruct();
+
+    ptr = p.try_get();
+    test_assert(ptr == nullptr);
+}
+
+
 void Refs_has(void) {
     flecs::world world;
 
@@ -226,3 +244,118 @@ void Refs_bool_operator(void) {
         test_assert(p);
     }
 }
+
+struct Base {
+    int x;
+};
+
+struct Derived : public Base {
+    Derived() { }
+
+    Derived(int x_, int y_) {
+        x = x_;
+        y = y_;
+    }
+
+    int y;
+};
+
+void Refs_base_type(void) {
+    flecs::world world;
+
+    flecs::entity e = world.entity().set(Derived{10, 20});
+
+    flecs::ref<Base> r = e.get_ref_w_id<Base>(world.id<Derived>());
+    test_int(r->x, 10);
+}
+
+struct BaseEmpty { };
+
+struct DerivedFromEmpty : public BaseEmpty {
+    DerivedFromEmpty() { }
+
+    DerivedFromEmpty(int y_) {
+        y = y_;
+    }
+
+    int y;
+};
+
+void Refs_empty_base_type(void) {
+    flecs::world world;
+
+    flecs::entity e = world.entity().set(DerivedFromEmpty{20});
+
+    flecs::ref<BaseEmpty> r = e.get_ref_w_id<BaseEmpty>(world.id<DerivedFromEmpty>());
+
+    BaseEmpty *b = r.get();
+    DerivedFromEmpty *d = static_cast<DerivedFromEmpty*>(b);
+    test_int(d->y, 20);
+}
+
+void Refs_get_component(void) {
+    flecs::world world;
+
+    auto e = flecs::entity(world)
+        .set<Position>({10, 20});
+
+    auto ref = e.get_ref<Position>();
+    test_assert(ref.component() == world.id<Position>());
+}
+
+void Refs_untyped_get_ref_by_method(void) {
+    flecs::world world;
+
+    auto e = flecs::entity(world)
+        .set<Position>({10, 20});
+
+    flecs::untyped_ref ref = e.get_ref(world.id<Position>());
+    Position* pos = static_cast<Position*>(ref.get());
+    test_assert(pos->x == 10);
+    test_assert(pos->y == 20);
+}
+
+void Refs_untyped_pair_ref(void) {
+    flecs::world world;
+
+    flecs::entity tag = world.entity();
+
+    auto e = world.entity().set<Position>(tag,{10, 20});
+    flecs::untyped_ref ref = e.get_ref(world.id<Position>(), tag);
+    Position* pos = static_cast<Position*>(ref.get());
+    pos->x ++;
+
+    test_int((e.get<Position>(tag)->x), 11);
+}
+
+void Refs_untyped_runtime_component_ref(void) {
+    flecs::world world;
+
+    auto Position = world.component("Position")
+        .member(flecs::I32, "x")
+        .member(flecs::I32, "y");
+
+    auto e = flecs::entity(world);
+
+    void* pos_ptr = e.ensure(Position);
+
+    flecs::cursor cur(world, Position, pos_ptr);
+    cur.push();
+    cur.member("x");
+    cur.set_int(10);
+    cur.member("y");
+    cur.set_int(20);
+    cur.pop();
+
+    flecs::untyped_ref ref = e.get_ref(Position);
+
+    cur = flecs::cursor(world, Position, ref.get());
+
+    cur.push();
+    cur.member("x");
+    test_int(cur.get_int(), 10);
+    cur.member("y");
+    test_int(cur.get_int(), 20);
+    cur.pop();
+
+}
\ No newline at end of file
