@@ -456,7 +456,8 @@ void SerializeQueryInfoToJson_1_tag_cascade(void) {
     ECS_ENTITY(world, Foo, (OnInstantiate, Inherit));
 
     ecs_query_t *q = ecs_query(world, {
-        .expr = "Foo(cascade)"
+        .expr = "Foo(cascade)",
+        .cache_kind = EcsQueryCacheAuto
     });
 
     test_assert(q != NULL);
@@ -710,7 +711,7 @@ void SerializeQueryInfoToJson_serialize_plan_trivial_query(void) {
     };
 
     char *json = ecs_iter_to_json(&it, &desc);
-    char *expect = flecs_asprintf("{\"query_plan\":null, \"results\":[]}");
+    char *expect = flecs_asprintf("{\"query_plan\":\"   [[0;37mOptimized out (trivial query)\\n\", \"results\":[]}");
     test_json(json, expect);
     ecs_os_free(json);
     ecs_os_free(expect);
